# -*- coding: utf-8 -*-

# This code is part of Qiskit.
#
# (C) Copyright IBM 2017, 2021.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.
"""This is the StarQubit module."""

from shapely.geometry import CAP_STYLE
from qiskit_metal import draw, Dict  # , QComponent
from qiskit_metal.qlibrary.core import QComponent


class StarQubit(QComponent):
    """A single configurable circle with multiple pads.

    Inherits `BaseQubit` class.

    Create a circular transmon qubit with up to 4 connectors and one readout.

    .. image::
        StarQubit.png

    .. meta::
        :description: Star Qubit

    Default Options:
        * radius: '300um' -- Radius of the circle defining the star shape
        * center_radius: '100um' -- Measure of how thick the central island is
        * gap_couplers: '25um' -- Gap between the star and the coupling resonator
        * gap_readout: '10um' -- Gap between the star and the readout resonator
        * connector_length: '75um' -- Length of the rectangular part of the connector
        * trap_offset: '20um' -- Offset between trapezoid coordinates for side wall angle
        * junc_h: '30um' -- Junction height
        * cpw_width='0.01', -- Junction width
        * rotation_cpl1: '0.0' -- Rotation for one of the coupling resonators '36.0', '0.0',
        * rotation_cpl2: '72.0' -- Rotation for the readout resonator '108.0','72.0',
        * rotation_rdout: '144.0' -- Rotation for one of the coupling resonators '180.0','144.0',
        * rotation_cpl3: '216.0' -- Rotation for one of the coupling resonators'252.0','216.0',
        * rotation_cpl4: '288.0' -- Rotation for one of the coupling resonators '324.0','288.0',
        * number_of_connectors: '4' -- Total number of coupling resonators
        * resolution: '16'
        * cap_style: 'round' -- round, flat, square
        * subtract: 'False'
        * helper: 'False'
    """
    component_metadata = Dict(short_name='Star',
                              _qgeometry_table_poly='True',
                              _qgeometry_table_junction='True')
    """Component metadata"""

    default_options = dict(radius='300um',
                           center_radius='100um',
                           gap_couplers='25um',
                           gap_readout='10um',
                           connector_length='75um',
                           trap_offset='20um',
                           junc_h='100um',
                           cpw_width='0.01',
                           rotation_cpl1='0.0',
                           rotation_cpl2='72.0',
                           rotation_rdout='144.0',
                           rotation_cpl3='216.0',
                           rotation_cpl4='288.0',
                           number_of_connectors='4',
                           resolution='16',
                           cap_style='round',
                           subtract='False',
                           helper='False')
    """Default drawing options"""

    def make(self):
        """The make function implements the logic that creates the geometry
        (poly, path, etc.) from the qcomponent.options dictionary of
        parameters, and the adds them to the design, using
        qcomponent.add_qgeometry(...), adding in extra needed information, such
        as layer, subtract, etc.
        """
        self.make_inner_star()
        self.make_coupling_resonators(4)
        self.make_readout_resonator()
        self.make_outer_circle()

    def make_circle(self):
        """This function creates a circle to be accessed later.
        """
        p = self.p
        # create a circle
        circle = draw.Point(0,
                            0).buffer(p.radius,
                                      resolution=int(p.resolution),
                                      cap_style=getattr(CAP_STYLE, p.cap_style))

        return circle

    def make_pockets(self):
        """This function creates the pockets.
        """
        p = self.p

        # Connector pocket width/height
        pocket_w = p.connector_length
        pocket_h = p.connector_length * 2
        # Connector to the JJ width/height
        pocket_w1 = p.gap_couplers
        pocket_h1 = p.center_radius
        pockets = [pocket_w, pocket_h, pocket_w1, pocket_h1]

        return pockets

    def make_rotation(self, obj, num):
        """This function rotates objects.
        """
        p = self.p
        if num == 1:
            rotation = [p.rotation_rdout]
            x = 1
        elif num == 2:
            rotation = [p.rotation_rdout + 180]
            x = 1
        elif num == 3:
            rotation = [
                p.rotation_cpl1 + 180, p.rotation_cpl2 + 180,
                p.rotation_cpl3 + 180, p.rotation_cpl4 + 180
            ]
            x = 4
        elif num == 4:
            rotation = [
                p.rotation_cpl1, p.rotation_cpl2, p.rotation_cpl3,
                p.rotation_cpl4
            ]
            x = 4
        elif num == 5:
            rotation = [
                p.rotation_cpl1, p.rotation_cpl2, p.rotation_rdout,
                p.rotation_cpl3, p.rotation_cpl4
            ]
            x = 5

        obj_array = [0] * x
        for i in range(x):
            obj_array[i] = draw.rotate(obj, rotation[i], origin=(0, 0))

        return obj_array

    def make_coordinates_trap(self):
        """This function creates the coordinates for trapezoid
        """
        p = self.p
        # Coordinates of the trapezoid to subtract from circle
        coord_y1 = -p.center_radius
        coord_y2 = -p.radius
        coord_x1 = -p.center_radius / 2
        coord_x2 = p.center_radius / 2
        coord_x3 = p.center_radius + p.trap_offset
        coord_x4 = -(p.center_radius + p.trap_offset)
        # coordinated for trapezoid to be cut out
        coords = [((coord_x1), (coord_y1)), ((coord_x2), (coord_y1)),
                  ((coord_x3), (coord_y2)), ((coord_x4), (coord_y2))]

        return coords

    def make_resonator_coordinates(self):
        """This function creates the coordinates for trapezoid
        """
        p = self.p

        # Coordinates of the polygon defining the coupling resonator (a=x, b=y)
        coord_a1 = -(p.center_radius + p.gap_couplers)
        coord_a2 = -p.radius
        coord_a3 = p.radius
        coord_b1 = -(p.center_radius / 4)
        coord_b2 = (p.center_radius / 4)
        coord_b3 = (p.center_radius - (p.gap_couplers))
        coord_b4 = -(p.center_radius - (p.gap_couplers))
        coord_b5 = -p.radius
        coord_b6 = p.radius

        # Define the coordinates of the polygon to define connectors
        coords_coupling_resonator = [((coord_b1), (coord_a1)),
                                     ((coord_b2), (coord_a1)),
                                     ((coord_b3), (coord_a2)),
                                     ((coord_b6), (coord_a2)),
                                     ((coord_b6), (coord_a3)),
                                     ((coord_b5), (coord_a3)),
                                     ((coord_b5), (coord_a2)),
                                     ((coord_b4), (coord_a2))]

        return coords_coupling_resonator

    def make_readout_coordinates(self):
        """This function creates the coordinates for trapezoid
        """
        p = self.p

        # Coordinates of the polygon defining the readout resonator (c=x, d=y)
        coord_c1 = -(p.center_radius + p.gap_readout)
        coord_c2 = -p.radius
        coord_c3 = p.radius
        coord_d1 = -(p.center_radius / 2 - p.gap_readout)
        coord_d2 = p.center_radius / 2 - p.gap_readout
        coord_d3 = p.center_radius
        coord_d4 = -p.center_radius
        coord_d5 = -p.radius
        coord_d6 = p.radius

        # Define the coordinates of the polygon to define readout
        coords_readout = [((coord_d1), (coord_c1)), ((coord_d2), (coord_c1)),
                          ((coord_d3), (coord_c2)), ((coord_d6), (coord_c2)),
                          ((coord_d6), (coord_c3)), ((coord_d5), (coord_c3)),
                          ((coord_d5), (coord_c2)), ((coord_d4), (coord_c2))]

        return coords_readout

    def make_pin_coordinates(self):
        """This function creates the coordinates for the pins
        """
        p = self.p
        p_in = (0, p.radius)
        p_out = (0, 1.25 * (p.radius))
        pins = draw.LineString([p_in, p_out])

        return pins

    def make_inner_star(self):
        """This function creates the coordinates for the pins
        """
        p = self.p
        # Extracting coordinated from the user input values
        coords = self.make_coordinates_trap()
        coords1 = self.make_resonator_coordinates()
        trap_0 = draw.Polygon(coords)
        traps = self.make_rotation(trap_0, 5)

        # Define the final structure based on use input
        if (p.number_of_connectors) == 0:
            traps = traps[2]
        elif (p.number_of_connectors) == 1:
            traps = draw.union(traps[0], traps[2])
        elif (p.number_of_connectors) == 2:
            traps = draw.union(traps[0], traps[1], traps[2])
        elif (p.number_of_connectors) == 3:
            traps = draw.union(traps[0], traps[1], traps[2], traps[3])
        elif (p.number_of_connectors) == 4:
            traps = draw.union(traps[0], traps[1], traps[2], traps[3], traps[4])

        # Subtract from circle
        circle = self.make_circle()
        total1 = draw.subtract(circle, traps)

        # create rectangular connectors to junction
        pockets = self.make_pockets()
        rect1 = draw.rectangle(pockets[2], pockets[3])
        rect1 = draw.translate(rect1, xoff=coords1[0][0] * 1.1, yoff=p.radius)
        rect1 = draw.rotate(rect1, p.rotation_cpl1, origin=(0, 0))
        rect2 = draw.rectangle(pockets[2], pockets[3])
        rect2 = draw.translate(rect2, xoff=coords1[1][0] * 1.1, yoff=p.radius)
        rect2 = draw.rotate(rect2, p.rotation_cpl1, origin=(0, 0))

        #junction
        jjunction = draw.LineString([[0, 0], [0, coords[1][0]]])
        jjunction = draw.translate(jjunction, yoff=(1.15 * (p.radius)))
        jjunction = draw.rotate(jjunction, p.rotation_cpl1, origin=(0, 0))

        # Add connection to the junction
        total = draw.union(total1, rect1, rect2)

        objects = [total, jjunction]
        objects = draw.rotate(objects, p.orientation, origin=(0, 0))
        objects = draw.translate(objects, p.pos_x, p.pos_y)
        [total, jjunction] = objects

        self.add_qgeometry('poly', {'circle_inner': total},
                           subtract=p.subtract,
                           helper=p.helper,
                           layer=p.layer,
                           chip=p.chip)

        self.add_qgeometry('junction', {'poly': jjunction},
                           subtract=p.subtract,
                           helper=p.helper,
                           layer=p.layer,
                           chip=p.chip,
                           width=p.junc_h)

    def make_coupling_resonators(self, num):
        """This function draws the coulping resonators.
           Adds pins. And adds the drawn geometry to qgeomtery table.
        """
        p = self.p
        # rotate these trapezoids to form the contacts

        coords = self.make_coordinates_trap()
        coords1 = self.make_resonator_coordinates()
        trap_z = draw.Polygon(coords1)
        traps_connection = self.make_rotation(trap_z, 4)

        # Define contacts
        pockets = self.make_pockets()
        pocket0 = draw.rectangle(pockets[0], pockets[1])
        pocket0 = draw.translate(pocket0, xoff=0, yoff=(coords[3][1]))
        pockets = self.make_rotation(pocket0, 4)

        # Define the connectors
        circle = self.make_circle()
        contacts = [0] * num
        for i in range(num):
            contacts[i] = draw.subtract(circle, traps_connection[i])
            contacts[i] = draw.union(contacts[i], pockets[i])

        pins = self.make_pin_coordinates()
        pins_cpl = self.make_rotation(pins, 3)

        objects = [contacts, pins_cpl]
        objects = draw.rotate(objects, p.orientation, origin=(0, 0))
        objects = draw.translate(objects, p.pos_x, p.pos_y)
        [contacts, pins_cpl] = objects

        ##################################################################
        # Add geometry and Qpin connections

        if (p.number_of_connectors) >= 1:
            self.add_qgeometry('poly', {'contact_cpl1': contacts[0]},
                               subtract=p.subtract,
                               helper=p.helper,
                               layer=p.layer,
                               chip=p.chip)
            # Add pin connections
            self.add_pin('pin_cpl1',
                         pins_cpl[0].coords,
                         width=p.cpw_width,
                         input_as_norm=True)
        if (p.number_of_connectors) >= 2:
            self.add_qgeometry('poly', {'contact_cpl2': contacts[1]},
                               subtract=p.subtract,
                               helper=p.helper,
                               layer=p.layer,
                               chip=p.chip)
            # Add pin connections
            self.add_pin('pin_cpl2',
                         pins_cpl[1].coords,
                         width=p.cpw_width,
                         input_as_norm=True)
        if (p.number_of_connectors) >= 3:
            self.add_qgeometry('poly', {'contact_cpl3': contacts[2]},
                               subtract=p.subtract,
                               helper=p.helper,
                               layer=p.layer,
                               chip=p.chip)
            # Add pin connections
            self.add_pin('pin_cpl3',
                         pins_cpl[2].coords,
                         width=p.cpw_width,
                         input_as_norm=True)
        if (p.number_of_connectors) >= 4:
            self.add_qgeometry('poly', {'contact_cpl4': contacts[3]},
                               subtract=p.subtract,
                               helper=p.helper,
                               layer=p.layer,
                               chip=p.chip)
            # Add pin connections
            self.add_pin('pin_cpl4',
                         pins_cpl[3].coords,
                         width=p.cpw_width,
                         input_as_norm=True)

    def make_readout_resonator(self):
        """This function draws the readout resonator.
           Adds pins. And adds the drawn geometry to qgeomtery table.
        """

        p = self.p
        coords_readout = self.make_readout_coordinates()
        circle = self.make_circle()
        pockets = self.make_pockets()
        coords = self.make_coordinates_trap()

        # Make the readout resonator with the pocket
        contact_rdout = draw.Polygon(coords_readout)
        contact_rdout = draw.subtract(circle, contact_rdout)
        contact_rdout = self.make_rotation(contact_rdout, 1)

        # Define contacts
        pocket0 = draw.rectangle(pockets[0], pockets[1])
        pocket0 = draw.translate(pocket0, xoff=0, yoff=(coords[3][1]))
        pocket0 = self.make_rotation(pocket0, 1)

        # Join the coupler and contact
        contact_rdout = draw.union(contact_rdout[0], pocket0[0])

        pins = self.make_pin_coordinates()
        pins_rdout = self.make_rotation(pins, 2)

        objects = [contact_rdout, pins_rdout]
        objects = draw.rotate(objects, p.orientation, origin=(0, 0))
        objects = draw.translate(objects, p.pos_x, p.pos_y)
        [contact_rdout, pins_rdout] = objects

        ##################################################################
        # Add geometry and Qpin connections

        self.add_qgeometry('poly', {'contact_rdout': contact_rdout},
                           subtract=p.subtract,
                           helper=p.helper,
                           layer=p.layer,
                           chip=p.chip)
        self.add_pin('pin_rdout',
                     pins_rdout[0].coords,
                     width=p.cpw_width,
                     input_as_norm=True)

    def make_outer_circle(self):
        """This function draws the outer circle.
        """

        p = self.p

        coords = self.make_coordinates_trap()

        circle_outer = draw.Point(0, 0).buffer(
            p.radius * (1 + (p.connector_length / p.radius)),
            resolution=int(p.resolution),
            cap_style=getattr(CAP_STYLE, p.cap_style))

        #Connectors for the ground plane
        pockets = self.make_pockets()
        pocket_z = draw.rectangle(pockets[0] * 1.4, pockets[1])
        pocket_z = draw.translate(pocket_z, xoff=0, yoff=(coords[2][1]))
        pockets_ground = self.make_rotation(pocket_z, 5)

        if (p.number_of_connectors) == 0:
            circle_outer = draw.union(circle_outer, pockets_ground[2])
        elif (p.number_of_connectors) == 1:
            circle_outer = draw.union(circle_outer, pockets_ground[0],
                                      pockets_ground[2])
        elif (p.number_of_connectors) == 2:
            circle_outer = draw.union(circle_outer, pockets_ground[0],
                                      pockets_ground[1], pockets_ground[2])
        elif (p.number_of_connectors) == 3:
            circle_outer = draw.union(circle_outer, pockets_ground[0],
                                      pockets_ground[1], pockets_ground[2],
                                      pockets_ground[3])
        elif (p.number_of_connectors) == 4:
            circle_outer = draw.union(circle_outer, pockets_ground[0],
                                      pockets_ground[1], pockets_ground[2],
                                      pockets_ground[3], pockets_ground[4])

        ##################################################################
        # Add geometry and Qpin connections
        objects = [circle_outer]
        objects = draw.rotate(objects, p.orientation, origin=(0, 0))
        objects = draw.translate(objects, p.pos_x, p.pos_y)
        [circle_outer] = objects
        self.add_qgeometry('poly', {'circle_outer': circle_outer},
                           subtract=True,
                           helper=p.helper,
                           layer=p.layer,
                           chip=p.chip)



# -*- coding: utf-8 -*-

# This code is part of Qiskit.
#
# (C) Copyright IBM 2017, 2021.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.
"""File contains dictionary for Rectangle and the make()."""

from qiskit_metal import draw, Dict
from qiskit_metal.qlibrary.core import QComponent


class Rectangle(QComponent):
    """A single configurable square.

    Inherits QComponent class.

    The class will add default_options class Dict to QComponent class before calling make.

    .. image::
        Rectangle.png

    .. meta::
        :description: Rectangle

    Default Options:
        * width: '500um'
        * height: '300um'
        * subtract: 'False'
        * helper: 'False'
    """

    default_options = Dict(width='500um',
                           height='300um',
                           subtract='False',
                           helper='False')
    """Default drawing options"""

    TOOLTIP = """A single configurable square"""

    def make(self):
        """The make function implements the logic that creates the geometry
        (poly, path, etc.) from the qcomponent.options dictionary of
        parameters, and the adds them to the design, using
        qcomponent.add_qgeometry(...), adding in extra needed information, such
        as layer, subtract, etc."""
        p = self.p  # p for parsed parameters. Access to the parsed options.

        # create the geometry
        rect = draw.rectangle(p.width, p.height, p.pos_x, p.pos_y)
        rect = draw.rotate(rect, p.orientation)
        ##############################################
        # add qgeometry
        self.add_qgeometry('poly', {'rectangle': rect},
                           subtract=p.subtract,
                           helper=p.helper,
                           layer=p.layer,
                           chip=p.chip)



# -*- coding: utf-8 -*-

# This code is part of Qiskit.
#
# (C) Copyright IBM 2017, 2021.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.

from abc import abstractmethod
from qiskit_metal.designs import QDesign  # pylint: disable=unused-import
from qiskit_metal import Dict
from qiskit_metal import config

from . import QAnalysis
import importlib


class QSimulation(QAnalysis):
    """A subclass of `QAnalysis`, intended to standardize across all Analysis classes
    select and name renderers.

    Default Setup:
        * name (str): Name of simulation setup. Defaults to "Setup".

    Data Labels:
        * sim_setup_name (str): Name given to the current setup.
    """

    default_setup = Dict(name="Setup",
                         reuse_selected_design=True,
                         reuse_setup=True)
    """Default setup"""

    # supported labels for data generated from the simulation
    data_labels = ['sim_setup_name']
    """Default data labels."""

    def __init__(self,
                 design: 'QDesign' = None,
                 renderer_name: str = None,
                 *args,
                 **kwargs):
        """Variables and method needed from all those Analysis types that need a renderer.

        Args:
            design (QDesign): The Metal design you are working on. Defaults to None.
            renderer_name (str): Name of the renderer you intend to use. Defaults to None.
        """
        super().__init__(*args, **kwargs)

        # pointer to find renderers
        self.design = design
        if self.design is None:
            self.logger.info(
                "You did not specify a design, so you will need to provide manual inputs"
                " for the analysis.")

        # verify renderer existence
        self.renderer_name = renderer_name
        self.renderer = None
        if self.renderer_name is None:
            self.logger.info(
                "You did not specify a renderer, so you are expected to manually provide "
                " the analysis input information.")
        else:
            self.renderer = self.select_renderer(renderer_name)

    def select_renderer(self, renderer_name: str):
        """Makes sure the renderer exists in qiskit-metal. If yes it sets the analysis 
        class variables to be able to reach it easily. Else it throws an error.

        Args:
            renderer_name (str): Name of the renderer you intend to use.

        Returns:
            (QRenderer): The renderer to be used in the analysis.
        """
        try:
            if self.design is None:
                # we want to setup a renderer from scratch

                # renderer_ref will be {} id renderer_name does not exist
                renderer_ref = config.renderers_to_load[renderer_name]
                if not renderer_ref:
                    raise KeyError  #needed because this is a Dict, not a dict
                if not (renderer_ref.path_name and renderer_ref.class_name):
                    self.logger.error(
                        f'The renderer={renderer_name} is not properly configured in'
                        ' config.renderers_to_load. Please add the missing information'
                        ' (Tip: needs to have both a path_name and a class_name keys).'
                    )
                    return None

                # if the path_name exists, grab the class
                if importlib.util.find_spec(renderer_ref.path_name):
                    class_renderer = getattr(
                        importlib.import_module(renderer_ref.path_name),
                        renderer_ref.class_name, None)

                    # if the class_name exists, then create the renderer object
                    if class_renderer is not None:
                        renderer = class_renderer(None)
                    else:
                        self.logger.warning(
                            f'Could not find the class={renderer_ref.class_name} '
                            f'in the renderer={renderer_name}')
                else:
                    self.logger.warning(
                        f'Could not find the renderer={renderer_name} '
                        f'at the path={renderer_ref.path_name}.')

            else:
                # the renderer would have been already registered within the design object
                renderer = self.design.renderers[renderer_name]
                if not renderer:
                    self.design.logger.error(
                        f"Cannot find the renderer \"{renderer_name}\" registered with qiskit-metal"
                    )
                    return None
        except KeyError:
            self.design.logger.error(
                f"Cannot find a renderer {renderer_name} registered with qiskit-metal"
            )
            return None
        return renderer

    def start(self):
        """Starts the renderer by executing the routine of the selected renderer.
        """
        self._initialize_renderer()

    def _initialize_renderer(self):
        """Starts the renderer by executing the routine of the selected renderer.
        """
        self.renderer.start()

    def _render(self, solution_type, vars_to_initialize,
                **design_selection) -> str:
        """Renders the design from qiskit metal into the selected renderer.
        First it decides the tentative name of the design. Then it runs the renderer method
        that executes the design rendering. It returns the final design name.

        Args:
            solution_type (str): The type of simulation solution to apply.
                Supported so far: eigenmode, capacitive, drivenmodal
            vars_to_initialize (Dict): The variables to initialize, i.e. Ljx, Cjx.

        Returns:
            (str): Final design name that the renderer used.
        """
        if self.design is None:
            return self.renderer.get_active_design_name()
        # need a default renderer-design name. Use the name of the metal-design.
        base_name = self.design.name
        # if a renderer-design name was provided as input to run(), use that as a base
        if "name" in design_selection:
            if design_selection["name"] is not None:
                base_name = design_selection["name"]
            del design_selection["name"]
        design_name = base_name + "_" + self.renderer_name
        design_name = self.renderer.execute_design(
            design_name,
            solution_type=solution_type,
            force_redraw=self.setup.reuse_selected_design,
            vars_to_initialize=vars_to_initialize,
            **design_selection)
        return design_name

    def close(self):
        """Stops the renderer by executing the routine of the selected renderer.
        """
        self._close_renderer()

    def _close_renderer(self):
        """Stops the renderer by executing the routine of the selected renderer.
        """
        self.renderer.stop()

    @property
    def renderer_initialized(self):
        """Reports whether the renderer is initialized or stopped.
        """
        return self.renderer.initialized

    def save_screenshot(self):
        """Saves the screenshot.

        Returns:
            (pathlib.WindowsPath): Path to png formatted screenshot.
        """
        return self.renderer.save_screenshot()

    def run(self, *args, **kwargs):
        """Alias for run_sim() necessary to implement super-class method, while
        preventing method name collision when sim and non-sim QAnalysis classes are inherited.
        """
        self.run_sim(*args, **kwargs)

    @abstractmethod
    def run_sim(self, *args, **kwargs):
        """Abstract method. Must be implemented by the subclass.
        Write in here the code to launch the simulations.
        You will be able to execute this with the alias run().
        """

    @property
    def sim_setup_name(self) -> str:
        """Getter

        Returns:
            str: Name of the setup being executed.
        """
        return self.get_data('sim_setup_name')

    @sim_setup_name.setter
    def sim_setup_name(self, data: str):
        """Setter

        Args:
            data (str): Name of the setup being executed.
        """
        if not isinstance(data, str):
            self.logger.warning(
                'Unsupported type %s. Only accepts str. Please try again.',
                {type(data)})
            return
        self.set_data('sim_setup_name', data)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './main_window_ui.ui',
# licensing of './main_window_ui.ui' applies.
#
# Created: Thu Jun 30 16:30:19 2022
#      by: pyside6-uic  running on PySide6 5.13.2
#
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):

    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1310, 847)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/metal_logo"), QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        MainWindow.setTabShape(QtWidgets.QTabWidget.Rounded)
        MainWindow.setDockNestingEnabled(True)
        MainWindow.setDockOptions(QtWidgets.QMainWindow.AllowNestedDocks |
                                  QtWidgets.QMainWindow.AllowTabbedDocks |
                                  QtWidgets.QMainWindow.AnimatedDocks |
                                  QtWidgets.QMainWindow.GroupedDragging)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setUsesScrollButtons(False)
        self.tabWidget.setMovable(False)
        self.tabWidget.setTabBarAutoHide(True)
        self.tabWidget.setObjectName("tabWidget")
        self.mainViewTab = QtWidgets.QWidget()
        self.mainViewTab.setObjectName("mainViewTab")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.mainViewTab)
        self.verticalLayout_9.setSpacing(0)
        self.verticalLayout_9.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/main_view"), QtGui.QIcon.Normal,
                        QtGui.QIcon.On)
        self.tabWidget.addTab(self.mainViewTab, icon1, "")
        self.tabQGeometry = QtWidgets.QWidget()
        self.tabQGeometry.setObjectName("tabQGeometry")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.tabQGeometry)
        self.verticalLayout_10.setSpacing(0)
        self.verticalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/elements"), QtGui.QIcon.Normal,
                        QtGui.QIcon.Off)
        self.tabWidget.addTab(self.tabQGeometry, icon2, "")
        self.tabNetList = QtWidgets.QWidget()
        self.tabNetList.setObjectName("tabNetList")
        self.verticalLayout_22 = QtWidgets.QVBoxLayout(self.tabNetList)
        self.verticalLayout_22.setSpacing(0)
        self.verticalLayout_22.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_22.setObjectName("verticalLayout_22")
        self.tabWidget.addTab(self.tabNetList, icon2, "")
        self.verticalLayout.addWidget(self.tabWidget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar()
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1310, 24))
        self.menubar.setBaseSize(QtCore.QSize(0, 0))
        self.menubar.setObjectName("menubar")
        self.menuDesign = QtWidgets.QMenu(self.menubar)
        self.menuDesign.setTearOffEnabled(False)
        self.menuDesign.setSeparatorsCollapsible(True)
        self.menuDesign.setToolTipsVisible(True)
        self.menuDesign.setObjectName("menuDesign")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        self.menuStylesheet = QtWidgets.QMenu(self.menuView)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/metal_logo"), QtGui.QIcon.Normal,
                        QtGui.QIcon.On)
        self.menuStylesheet.setIcon(icon3)
        self.menuStylesheet.setObjectName("menuStylesheet")
        self.menuAnalysis = QtWidgets.QMenu(self.menubar)
        self.menuAnalysis.setObjectName("menuAnalysis")
        self.menuRender = QtWidgets.QMenu(self.menubar)
        self.menuRender.setObjectName("menuRender")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setMinimumSize(QtCore.QSize(0, 0))
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/help"), QtGui.QIcon.Normal,
                        QtGui.QIcon.On)
        self.menuHelp.setIcon(icon4)
        self.menuHelp.setToolTipsVisible(True)
        self.menuHelp.setObjectName("menuHelp")
        self.menuPlot = QtWidgets.QMenu(self.menubar)
        self.menuPlot.setObjectName("menuPlot")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.toolBarDesign = QtWidgets.QToolBar(MainWindow)
        self.toolBarDesign.setOrientation(QtCore.Qt.Horizontal)
        self.toolBarDesign.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.toolBarDesign.setObjectName("toolBarDesign")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBarDesign)
        self.toolBarView = QtWidgets.QToolBar(MainWindow)
        font = QtGui.QFont()
        font.setLegacyWeight(75)
        font.setBold(True)
        self.toolBarView.setFont(font)
        self.toolBarView.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        self.toolBarView.setObjectName("toolBarView")
        MainWindow.addToolBar(QtCore.Qt.LeftToolBarArea, self.toolBarView)
        self.dockDesign = QtWidgets.QDockWidget(MainWindow)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/design"), QtGui.QIcon.Normal,
                        QtGui.QIcon.On)
        self.dockDesign.setWindowIcon(icon5)
        self.dockDesign.setFloating(False)
        # self.dockDesign.setFeatures(QtWidgets.QDockWidget.DockWidgetFeatures)
        self.dockDesign.setObjectName("dockDesign")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setSizeConstraint(QtWidgets.QLayout.SetNoConstraint)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setContentsMargins(-1, 2, -1, 2)
        self.gridLayout.setObjectName("gridLayout")
        self.btn_comp_zoom = QtWidgets.QToolButton(self.dockWidgetContents)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/fullscreen"), QtGui.QIcon.Normal,
                        QtGui.QIcon.Off)
        self.btn_comp_zoom.setIcon(icon6)
        self.btn_comp_zoom.setObjectName("btn_comp_zoom")
        self.gridLayout.addWidget(self.btn_comp_zoom, 0, 2, 1, 1)
        self.btn_comp_del = QtWidgets.QToolButton(self.dockWidgetContents)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/delete"), QtGui.QIcon.Normal,
                        QtGui.QIcon.Off)
        self.btn_comp_del.setIcon(icon7)
        self.btn_comp_del.setPopupMode(QtWidgets.QToolButton.DelayedPopup)
        self.btn_comp_del.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        self.btn_comp_del.setObjectName("btn_comp_del")
        self.gridLayout.addWidget(self.btn_comp_del, 0, 1, 1, 1)
        self.btn_comp_rename = QtWidgets.QToolButton(self.dockWidgetContents)
        self.btn_comp_rename.setObjectName("btn_comp_rename")
        self.gridLayout.addWidget(self.btn_comp_rename, 0, 0, 1, 1)
        self.verticalLayout_3.addLayout(self.gridLayout)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.filter_text_design = QtWidgets.QLineEdit(self.dockWidgetContents)
        self.filter_text_design.setText("")
        self.filter_text_design.setClearButtonEnabled(True)
        self.filter_text_design.setObjectName("filter_text_design")
        self.horizontalLayout.addWidget(self.filter_text_design)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.tableComponents = QTableView_AllComponents(self.dockWidgetContents)
        self.tableComponents.setSizeAdjustPolicy(
            QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.tableComponents.setAutoScroll(False)
        self.tableComponents.setAlternatingRowColors(True)
        self.tableComponents.setSelectionBehavior(
            QtWidgets.QAbstractItemView.SelectRows)
        self.tableComponents.setSortingEnabled(True)
        self.tableComponents.setObjectName("tableComponents")
        self.tableComponents.horizontalHeader().setVisible(True)
        self.tableComponents.horizontalHeader().setSortIndicatorShown(False)
        self.tableComponents.horizontalHeader().setStretchLastSection(True)
        self.tableComponents.verticalHeader().setVisible(False)
        self.verticalLayout_3.addWidget(self.tableComponents)
        self.dockDesign.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockDesign)
        self.dockLibrary = QtWidgets.QDockWidget(MainWindow)
        self.dockLibrary.setMinimumSize(QtCore.QSize(79, 124))
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/component"), QtGui.QIcon.Normal,
                        QtGui.QIcon.On)
        self.dockLibrary.setWindowIcon(icon8)
        # self.dockLibrary.setFeatures(
        #     QtWidgets.QDockWidget.AllDockWidgetFeatures)
        self.dockLibrary.setObjectName("dockLibrary")
        self.dockLibraryContents = QtWidgets.QWidget()
        self.dockLibraryContents.setObjectName("dockLibraryContents")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.dockLibraryContents)
        self.verticalLayout_5.setSpacing(0)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.dockLibrary_filter = QtWidgets.QLineEdit(self.dockLibraryContents)
        self.dockLibrary_filter.setObjectName("dockLibrary_filter")
        self.verticalLayout_5.addWidget(self.dockLibrary_filter)
        self.dockLibrary_tree_view = TreeViewQLibrary(self.dockLibraryContents)
        self.dockLibrary_tree_view.setObjectName("dockLibrary_tree_view")
        self.verticalLayout_5.addWidget(self.dockLibrary_tree_view)
        self.dockLibrary.setWidget(self.dockLibraryContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockLibrary)
        self.dockComponent = QtWidgets.QDockWidget(MainWindow)
        self.dockComponent.setMinimumSize(QtCore.QSize(62, 38))
        # self.dockComponent.setFeatures(
        #     QtWidgets.QDockWidget.AllDockWidgetFeatures)
        self.dockComponent.setObjectName("dockComponent")
        self.dockComponentCental = QtWidgets.QWidget()
        self.dockComponentCental.setObjectName("dockComponentCental")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.dockComponentCental)
        self.verticalLayout_7.setSpacing(0)
        self.verticalLayout_7.setContentsMargins(0, 10, 0, 0)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.dockComponent.setWidget(self.dockComponentCental)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1),
                                 self.dockComponent)
        self.dockLog = QtWidgets.QDockWidget(MainWindow)
        self.dockLog.setMinimumSize(QtCore.QSize(79, 101))
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/log"), QtGui.QIcon.Normal,
                        QtGui.QIcon.On)
        self.dockLog.setWindowIcon(icon9)
        # self.dockLog.setFeatures(QtWidgets.QDockWidget.AllDockWidgetFeatures)
        self.dockLog.setObjectName("dockLog")
        self.dockWidgetContents_4 = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred,
                                           QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.dockWidgetContents_4.sizePolicy().hasHeightForWidth())
        self.dockWidgetContents_4.setSizePolicy(sizePolicy)
        self.dockWidgetContents_4.setObjectName("dockWidgetContents_4")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.dockWidgetContents_4)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setSizeConstraint(
            QtWidgets.QLayout.SetMaximumSize)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.log_text = QTextEditLogger(self.dockWidgetContents_4)
        self.log_text.setTextInteractionFlags(
            QtCore.Qt.LinksAccessibleByKeyboard |
            QtCore.Qt.LinksAccessibleByMouse |
            QtCore.Qt.TextBrowserInteraction |
            QtCore.Qt.TextSelectableByKeyboard |
            QtCore.Qt.TextSelectableByMouse)
        self.log_text.setObjectName("log_text")
        self.verticalLayout_4.addWidget(self.log_text)
        self.dockLog.setWidget(self.dockWidgetContents_4)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(8), self.dockLog)
        self.dockConnectors = QtWidgets.QDockWidget(MainWindow)
        self.dockConnectors.setMinimumSize(QtCore.QSize(79, 124))
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/connectors"), QtGui.QIcon.Normal,
                         QtGui.QIcon.On)
        self.dockConnectors.setWindowIcon(icon10)
        self.dockConnectors.setObjectName("dockConnectors")
        self.dockWidgetContents_5 = QtWidgets.QWidget()
        self.dockWidgetContents_5.setObjectName("dockWidgetContents_5")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.dockWidgetContents_5)
        self.verticalLayout_6.setSpacing(0)
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.text_filter_connectors = QtWidgets.QLineEdit(
            self.dockWidgetContents_5)
        self.text_filter_connectors.setClearButtonEnabled(True)
        self.text_filter_connectors.setObjectName("text_filter_connectors")
        self.horizontalLayout_3.addWidget(self.text_filter_connectors)
        self.verticalLayout_6.addLayout(self.horizontalLayout_3)
        self.tableConnectors = QtWidgets.QTableView(self.dockWidgetContents_5)
        self.tableConnectors.setFrameShadow(QtWidgets.QFrame.Plain)
        self.tableConnectors.setSizeAdjustPolicy(
            QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.tableConnectors.setAutoScroll(False)
        self.tableConnectors.setAlternatingRowColors(False)
        self.tableConnectors.setObjectName("tableConnectors")
        self.tableConnectors.horizontalHeader().setVisible(False)
        self.verticalLayout_6.addWidget(self.tableConnectors)
        self.dockConnectors.setWidget(self.dockWidgetContents_5)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1),
                                 self.dockConnectors)
        self.dockVariables = QtWidgets.QDockWidget(MainWindow)
        self.dockVariables.setObjectName("dockVariables")
        self.dockVariablesContents = QtWidgets.QWidget()
        self.dockVariablesContents.setObjectName("dockVariablesContents")
        self.vlayoutVariables = QtWidgets.QVBoxLayout(
            self.dockVariablesContents)
        self.vlayoutVariables.setSpacing(0)
        self.vlayoutVariables.setContentsMargins(0, 0, 0, 0)
        self.vlayoutVariables.setObjectName("vlayoutVariables")
        self.dockVariables.setWidget(self.dockVariablesContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1),
                                 self.dockVariables)
        self.toolbar_renderers = QtWidgets.QToolBar(MainWindow)
        self.toolbar_renderers.setIconSize(QtCore.QSize(32, 32))
        self.toolbar_renderers.setObjectName("toolbar_renderers")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolbar_renderers)
        self.actionSave = QtGui.QAction(MainWindow)
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/save"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.actionSave.setIcon(icon11)
        self.actionSave.setObjectName("actionSave")
        self.actionLoad = QtGui.QAction(MainWindow)
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/load"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.actionLoad.setIcon(icon12)
        self.actionLoad.setAutoRepeat(False)
        self.actionLoad.setObjectName("actionLoad")
        self.actionDesign = QtGui.QAction(MainWindow)
        self.actionDesign.setCheckable(True)
        self.actionDesign.setChecked(True)
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(":/design"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.actionDesign.setIcon(icon13)
        self.actionDesign.setObjectName("actionDesign")
        self.actionElements = QtGui.QAction(MainWindow)
        self.actionElements.setCheckable(True)
        self.actionElements.setChecked(False)
        self.actionElements.setIcon(icon2)
        self.actionElements.setObjectName("actionElements")
        self.actionLog = QtGui.QAction(MainWindow)
        self.actionLog.setCheckable(True)
        self.actionLog.setChecked(True)
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(":/log"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.actionLog.setIcon(icon14)
        self.actionLog.setObjectName("actionLog")
        self.actionNewComponent = QtGui.QAction(MainWindow)
        self.actionNewComponent.setCheckable(True)
        self.actionNewComponent.setChecked(False)
        icon15 = QtGui.QIcon()
        icon15.addPixmap(QtGui.QPixmap(":/component-library"),
                         QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionNewComponent.setIcon(icon15)
        self.actionNewComponent.setObjectName("actionNewComponent")
        self.actionDelete_All = QtGui.QAction(MainWindow)
        icon16 = QtGui.QIcon()
        icon16.addPixmap(QtGui.QPixmap(":/delete_all"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.actionDelete_All.setIcon(icon16)
        self.actionDelete_All.setPriority(QtGui.QAction.LowPriority)
        self.actionDelete_All.setObjectName("actionDelete_All")
        self.actionZoom = QtGui.QAction(MainWindow)
        icon17 = QtGui.QIcon()
        icon17.addPixmap(QtGui.QPixmap(":/plot/zoom"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.actionZoom.setIcon(icon17)
        self.actionZoom.setObjectName("actionZoom")
        self.actionPan = QtGui.QAction(MainWindow)
        icon18 = QtGui.QIcon()
        icon18.addPixmap(QtGui.QPixmap(":/plot/pan"), QtGui.QIcon.Normal,
                         QtGui.QIcon.On)
        self.actionPan.setIcon(icon18)
        self.actionPan.setObjectName("actionPan")
        self.actionConnectors = QtGui.QAction(MainWindow)
        self.actionConnectors.setCheckable(True)
        self.actionConnectors.setChecked(False)
        icon19 = QtGui.QIcon()
        icon19.addPixmap(QtGui.QPixmap(":/connectors"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.actionConnectors.setIcon(icon19)
        self.actionConnectors.setObjectName("actionConnectors")
        self.actionStyleOpen = QtGui.QAction(MainWindow)
        self.actionStyleOpen.setObjectName("actionStyleOpen")
        self.actionStyleDefault = QtGui.QAction(MainWindow)
        self.actionStyleDefault.setObjectName("actionStyleDefault")
        self.actionStyleDark = QtGui.QAction(MainWindow)
        self.actionStyleDark.setObjectName("actionStyleDark")
        self.actionScreenshot = QtGui.QAction(MainWindow)
        icon20 = QtGui.QIcon()
        icon20.addPixmap(QtGui.QPixmap(":/screenshot"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.actionScreenshot.setIcon(icon20)
        self.actionScreenshot.setObjectName("actionScreenshot")
        self.action_full_refresh = QtGui.QAction(MainWindow)
        icon21 = QtGui.QIcon()
        icon21.addPixmap(QtGui.QPixmap(":/force_refresh"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.action_full_refresh.setIcon(icon21)
        self.action_full_refresh.setShortcutContext(
            QtCore.Qt.WidgetWithChildrenShortcut)
        self.action_full_refresh.setAutoRepeat(False)
        self.action_full_refresh.setPriority(QtGui.QAction.HighPriority)
        self.action_full_refresh.setObjectName("action_full_refresh")
        self.actionRebuild = QtGui.QAction(MainWindow)
        icon22 = QtGui.QIcon()
        icon22.addPixmap(QtGui.QPixmap(":/rebuild"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        icon22.addPixmap(QtGui.QPixmap(":/rebuild"), QtGui.QIcon.Active,
                         QtGui.QIcon.On)
        self.actionRebuild.setIcon(icon22)
        self.actionRebuild.setPriority(QtGui.QAction.HighPriority)
        self.actionRebuild.setObjectName("actionRebuild")
        self.actionFull_Screen = QtGui.QAction(MainWindow)
        icon23 = QtGui.QIcon()
        icon23.addPixmap(QtGui.QPixmap(":/plot/----autozoom"),
                         QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionFull_Screen.setIcon(icon23)
        self.actionFull_Screen.setObjectName("actionFull_Screen")
        self.actionMetal_Dark = QtGui.QAction(MainWindow)
        self.actionMetal_Dark.setObjectName("actionMetal_Dark")
        self.actionSaveAs = QtGui.QAction(MainWindow)
        self.actionSaveAs.setIcon(icon11)
        self.actionSaveAs.setObjectName("actionSaveAs")
        self.actionSave_window_state = QtGui.QAction(MainWindow)
        self.actionSave_window_state.setIcon(icon11)
        self.actionSave_window_state.setObjectName("actionSave_window_state")
        self.actionLabelDesign = QtGui.QAction(MainWindow)
        self.actionLabelDesign.setEnabled(False)
        self.actionLabelDesign.setObjectName("actionLabelDesign")
        self.actionClose_window = QtGui.QAction(MainWindow)
        self.actionClose_window.setObjectName("actionClose_window")
        self.actionMetal_Window = QtGui.QAction(MainWindow)
        self.actionMetal_Window.setEnabled(False)
        self.actionMetal_Window.setObjectName("actionMetal_Window")
        self.actionViewDummyLabel = QtGui.QAction(MainWindow)
        self.actionViewDummyLabel.setObjectName("actionViewDummyLabel")
        self.actionVariables = QtGui.QAction(MainWindow)
        self.actionVariables.setCheckable(True)
        self.actionVariables.setChecked(False)
        icon24 = QtGui.QIcon()
        icon24.addPixmap(QtGui.QPixmap(":/variables"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.actionVariables.setIcon(icon24)
        self.actionVariables.setObjectName("actionVariables")
        self.actionToggleDocks = QtGui.QAction(MainWindow)
        self.actionToggleDocks.setIcon(icon6)
        self.actionToggleDocks.setObjectName("actionToggleDocks")
        self.actionGDS = QtGui.QAction(MainWindow)
        icon25 = QtGui.QIcon()
        icon25.addPixmap(QtGui.QPixmap(":/renderer/_imgs/renderers/GDS.png"),
                         QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionGDS.setIcon(icon25)
        self.actionGDS.setObjectName("actionGDS")
        self.actionHFSS = QtGui.QAction(MainWindow)
        icon26 = QtGui.QIcon()
        icon26.addPixmap(QtGui.QPixmap(":/renderer/_imgs/renderers/HFSS.png"),
                         QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionHFSS.setIcon(icon26)
        self.actionHFSS.setObjectName("actionHFSS")
        self.actionQ3D = QtGui.QAction(MainWindow)
        icon27 = QtGui.QIcon()
        icon27.addPixmap(QtGui.QPixmap(":/renderer/_imgs/renderers/Q3D.png"),
                         QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionQ3D.setIcon(icon27)
        self.actionQ3D.setObjectName("actionQ3D")
        self.actionBuildHistory = QtGui.QAction(MainWindow)
        icon28 = QtGui.QIcon()
        icon28.addPixmap(QtGui.QPixmap(":/build_history"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.actionBuildHistory.setIcon(icon28)
        self.actionBuildHistory.setObjectName("actionBuildHistory")
        self.actionDeveloperMode = QtGui.QAction(MainWindow)
        self.actionDeveloperMode.setCheckable(True)
        icon29 = QtGui.QIcon()
        icon29.addPixmap(QtGui.QPixmap(":/_imgs/lightbulb.png"),
                         QtGui.QIcon.Normal, QtGui.QIcon.Off)
        icon29.addPixmap(QtGui.QPixmap(":/_imgs/support.png"),
                         QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionDeveloperMode.setIcon(icon29)
        self.actionDeveloperMode.setObjectName("actionDeveloperMode")
        self.actionWebHelp = QtGui.QAction(MainWindow)
        icon30 = QtGui.QIcon()
        icon30.addPixmap(QtGui.QPixmap(":/browser"), QtGui.QIcon.Normal,
                         QtGui.QIcon.Off)
        self.actionWebHelp.setIcon(icon30)
        self.actionWebHelp.setObjectName("actionWebHelp")
        self.menuDesign.addSeparator()
        self.menuDesign.addAction(self.actionLabelDesign)
        self.menuDesign.addAction(self.actionLoad)
        self.menuDesign.addAction(self.actionSave)
        self.menuDesign.addAction(self.actionSaveAs)
        self.menuDesign.addSeparator()
        self.menuDesign.addAction(self.actionDelete_All)
        self.menuDesign.addSeparator()
        self.menuDesign.addAction(self.actionClose_window)
        self.menuStylesheet.addAction(self.actionMetal_Dark)
        self.menuStylesheet.addAction(self.actionStyleDark)
        self.menuStylesheet.addSeparator()
        self.menuStylesheet.addAction(self.actionStyleDefault)
        self.menuStylesheet.addAction(self.actionStyleOpen)
        self.menuView.addAction(self.actionDesign)
        self.menuView.addAction(self.actionElements)
        self.menuView.addAction(self.actionConnectors)
        self.menuView.addAction(self.actionLog)
        self.menuView.addAction(self.actionNewComponent)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionToggleDocks)
        self.menuView.addAction(self.actionFull_Screen)
        self.menuView.addAction(self.actionScreenshot)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionMetal_Window)
        self.menuView.addAction(self.actionSave_window_state)
        self.menuView.addAction(self.menuStylesheet.menuAction())
        self.menuAnalysis.addSeparator()
        self.menuRender.addSeparator()
        self.menuPlot.addAction(self.actionPan)
        self.menuPlot.addAction(self.actionZoom)
        self.menubar.addAction(self.menuDesign.menuAction())
        self.menubar.addAction(self.menuPlot.menuAction())
        self.menubar.addAction(self.menuRender.menuAction())
        self.menubar.addAction(self.menuAnalysis.menuAction())
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())
        self.toolBarDesign.addAction(self.actionLoad)
        self.toolBarDesign.addAction(self.actionSave)
        self.toolBarDesign.addAction(self.actionDelete_All)
        self.toolBarDesign.addSeparator()
        self.toolBarDesign.addAction(self.action_full_refresh)
        self.toolBarDesign.addAction(self.actionRebuild)
        self.toolBarDesign.addAction(self.actionBuildHistory)
        self.toolBarDesign.addSeparator()
        self.toolBarDesign.addAction(self.actionWebHelp)
        self.toolBarDesign.addSeparator()
        self.toolBarView.addAction(self.actionToggleDocks)
        self.toolBarView.addAction(self.actionScreenshot)
        self.toolbar_renderers.addAction(self.actionGDS)
        self.toolbar_renderers.addAction(self.actionHFSS)
        self.toolbar_renderers.addAction(self.actionQ3D)

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(2)
        QtCore.QObject.connect(self.actionElements,
                               QtCore.SIGNAL("toggled(bool)"),
                               MainWindow._set_element_tab)
        QtCore.QObject.connect(self.actionDelete_All,
                               QtCore.SIGNAL("triggered()"),
                               MainWindow.delete_all_components)
        QtCore.QObject.connect(self.actionLoad, QtCore.SIGNAL("triggered()"),
                               MainWindow.load_design)
        QtCore.QObject.connect(self.actionSave, QtCore.SIGNAL("triggered()"),
                               MainWindow.save_design)
        QtCore.QObject.connect(self.actionStyleDark,
                               QtCore.SIGNAL("triggered()"),
                               MainWindow.load_stylesheet_dark)
        QtCore.QObject.connect(self.actionStyleDefault,
                               QtCore.SIGNAL("triggered()"),
                               MainWindow.load_stylesheet_default)
        QtCore.QObject.connect(self.actionScreenshot,
                               QtCore.SIGNAL("triggered()"),
                               MainWindow._screenshot)
        QtCore.QObject.connect(self.actionStyleOpen,
                               QtCore.SIGNAL("triggered()"),
                               MainWindow.load_stylesheet_open)
        QtCore.QObject.connect(self.action_full_refresh,
                               QtCore.SIGNAL("triggered()"),
                               MainWindow.full_refresh)
        QtCore.QObject.connect(self.actionRebuild, QtCore.SIGNAL("triggered()"),
                               MainWindow.rebuild)
        QtCore.QObject.connect(self.actionFull_Screen,
                               QtCore.SIGNAL("triggered()"),
                               MainWindow.showFullScreen)
        QtCore.QObject.connect(self.actionMetal_Dark,
                               QtCore.SIGNAL("triggered()"),
                               MainWindow.load_stylesheet_metal_dark)
        QtCore.QObject.connect(self.dockDesign,
                               QtCore.SIGNAL("visibilityChanged(bool)"),
                               self.actionDesign.setChecked)
        QtCore.QObject.connect(self.actionDesign,
                               QtCore.SIGNAL("triggered(bool)"),
                               self.dockDesign.show)
        QtCore.QObject.connect(self.dockConnectors,
                               QtCore.SIGNAL("visibilityChanged(bool)"),
                               self.actionConnectors.setChecked)
        QtCore.QObject.connect(self.dockLog,
                               QtCore.SIGNAL("visibilityChanged(bool)"),
                               self.actionLog.setChecked)
        QtCore.QObject.connect(self.dockLibrary,
                               QtCore.SIGNAL("visibilityChanged(bool)"),
                               self.actionNewComponent.setChecked)
        QtCore.QObject.connect(self.actionNewComponent,
                               QtCore.SIGNAL("triggered(bool)"),
                               self.dockLibrary.show)
        QtCore.QObject.connect(self.actionLog, QtCore.SIGNAL("triggered(bool)"),
                               self.dockLog.show)
        QtCore.QObject.connect(self.actionConnectors,
                               QtCore.SIGNAL("triggered(bool)"),
                               self.dockConnectors.show)
        QtCore.QObject.connect(self.actionDesign, QtCore.SIGNAL("triggered()"),
                               self.dockDesign.raise_)
        QtCore.QObject.connect(self.actionConnectors,
                               QtCore.SIGNAL("triggered()"),
                               self.dockConnectors.raise_)
        QtCore.QObject.connect(self.actionNewComponent,
                               QtCore.SIGNAL("triggered()"),
                               self.dockLibrary.raise_)
        QtCore.QObject.connect(self.actionElements,
                               QtCore.SIGNAL("triggered()"),
                               self.tabWidget.setFocus)
        QtCore.QObject.connect(self.actionLog, QtCore.SIGNAL("triggered()"),
                               self.dockLog.raise_)
        QtCore.QObject.connect(self.actionSaveAs, QtCore.SIGNAL("triggered()"),
                               MainWindow.save_design_copy)
        QtCore.QObject.connect(self.actionClose_window,
                               QtCore.SIGNAL("triggered()"), MainWindow.close)
        QtCore.QObject.connect(self.actionSave_window_state,
                               QtCore.SIGNAL("triggered()"),
                               MainWindow.save_window_settings)
        QtCore.QObject.connect(self.actionVariables,
                               QtCore.SIGNAL("triggered(bool)"),
                               self.dockVariables.show)
        QtCore.QObject.connect(self.actionVariables,
                               QtCore.SIGNAL("triggered()"),
                               self.dockVariables.raise_)
        QtCore.QObject.connect(self.actionToggleDocks,
                               QtCore.SIGNAL("triggered()"),
                               MainWindow.toggle_all_docks)
        QtCore.QObject.connect(self.actionBuildHistory,
                               QtCore.SIGNAL("triggered()"),
                               MainWindow.create_build_log_window)
        QtCore.QObject.connect(self.actionGDS, QtCore.SIGNAL("triggered()"),
                               MainWindow.show_renderer_gds)
        QtCore.QObject.connect(self.actionHFSS, QtCore.SIGNAL("triggered()"),
                               MainWindow.show_renderer_hfss)
        QtCore.QObject.connect(self.actionQ3D, QtCore.SIGNAL("triggered()"),
                               MainWindow.show_renderer_q3d)
        QtCore.QObject.connect(self.actionWebHelp, QtCore.SIGNAL("triggered()"),
                               MainWindow.open_web_help)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Qiskit Metal: Quantum Creator",
                                             None, -1))
        MainWindow.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Qiskit Metal: Quantum Creator",
                                             None, -1))
        MainWindow.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Qiskit Metal: Quantum Creator",
                                             None, -1))
        MainWindow.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Qiskit Metal: Quantum Creator",
                                             None, -1))
        self.tabWidget.setTabText(
            self.tabWidget.indexOf(self.mainViewTab),
            QtWidgets.QApplication.translate("MainWindow", "Main View", None,
                                             -1))
        self.tabWidget.setTabText(
            self.tabWidget.indexOf(self.tabQGeometry),
            QtWidgets.QApplication.translate("MainWindow", "QGeometry", None,
                                             -1))
        self.tabWidget.setTabToolTip(
            self.tabWidget.indexOf(self.tabQGeometry),
            QtWidgets.QApplication.translate(
                "MainWindow", "Show the QGeometry elements of the design", None,
                -1))
        self.tabWidget.setTabText(
            self.tabWidget.indexOf(self.tabNetList),
            QtWidgets.QApplication.translate("MainWindow", "Net List", None,
                                             -1))
        self.tabWidget.setTabToolTip(
            self.tabWidget.indexOf(self.tabNetList),
            QtWidgets.QApplication.translate("MainWindow", "Show the Net List",
                                             None, -1))
        self.menuDesign.setToolTip(
            QtWidgets.QApplication.translate("MainWindow", "Design Menu", None,
                                             -1))
        self.menuDesign.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow", "Design Menu", None,
                                             -1))
        self.menuDesign.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow", "Design Menu", None,
                                             -1))
        self.menuDesign.setTitle(
            QtWidgets.QApplication.translate("MainWindow", "Metal Design", None,
                                             -1))
        self.menuView.setTitle(
            QtWidgets.QApplication.translate("MainWindow", "Window View", None,
                                             -1))
        self.menuStylesheet.setTitle(
            QtWidgets.QApplication.translate("MainWindow", "Color theme", None,
                                             -1))
        self.menuAnalysis.setTitle(
            QtWidgets.QApplication.translate("MainWindow", "Analysis", None,
                                             -1))
        self.menuRender.setTitle(
            QtWidgets.QApplication.translate("MainWindow", "Render", None, -1))
        self.menuHelp.setToolTip(
            QtWidgets.QApplication.translate("MainWindow", "Open Metal Help",
                                             None, -1))
        self.menuHelp.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow", "Open Metal Help",
                                             None, -1))
        self.menuHelp.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow", "Open Metal Help",
                                             None, -1))
        self.menuHelp.setTitle(
            QtWidgets.QApplication.translate("MainWindow", "Help", None, -1))
        self.menuPlot.setTitle(
            QtWidgets.QApplication.translate("MainWindow", "Plot", None, -1))
        self.toolBarDesign.setWindowTitle(
            QtWidgets.QApplication.translate("MainWindow", "Design toolbar",
                                             None, -1))
        self.toolBarDesign.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow", "Design toolbar",
                                             None, -1))
        self.toolBarDesign.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow", "Design toolbar",
                                             None, -1))
        self.toolBarView.setWindowTitle(
            QtWidgets.QApplication.translate("MainWindow", "View Toolbar", None,
                                             -1))
        self.toolBarView.setToolTip(
            QtWidgets.QApplication.translate("MainWindow", "View Toolbar", None,
                                             -1))
        self.toolBarView.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow", "View Toolbar", None,
                                             -1))
        self.toolBarView.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow", "View Toolbar", None,
                                             -1))
        self.dockDesign.setToolTip(
            QtWidgets.QApplication.translate("MainWindow", "Design Dock", None,
                                             -1))
        self.dockDesign.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow", "Design Dock", None,
                                             -1))
        self.dockDesign.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow", "Design Dock", None,
                                             -1))
        self.dockDesign.setAccessibleName(
            QtWidgets.QApplication.translate("MainWindow", "Design Dock", None,
                                             -1))
        self.dockDesign.setAccessibleDescription(
            QtWidgets.QApplication.translate("MainWindow", "Design Dock", None,
                                             -1))
        self.dockDesign.setWindowTitle(
            QtWidgets.QApplication.translate("MainWindow", "QComponents", None,
                                             -1))
        self.btn_comp_zoom.setToolTip(
            QtWidgets.QApplication.translate(
                "MainWindow", "Focus on component in drawing window", None, -1))
        self.btn_comp_zoom.setStatusTip(
            QtWidgets.QApplication.translate(
                "MainWindow", "Focus on component in drawing window", None, -1))
        self.btn_comp_zoom.setText(
            QtWidgets.QApplication.translate(
                "MainWindow", "Focus on component in drawing window", None, -1))
        self.btn_comp_del.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Delete a selected component",
                                             None, -1))
        self.btn_comp_del.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Delete a selected component",
                                             None, -1))
        self.btn_comp_del.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Delete a selected component",
                                             None, -1))
        self.btn_comp_del.setText(
            QtWidgets.QApplication.translate("MainWindow", "Delete", None, -1))
        self.btn_comp_rename.setText(
            QtWidgets.QApplication.translate("MainWindow", "Rename", None, -1))
        self.filter_text_design.setPlaceholderText(
            QtWidgets.QApplication.translate("MainWindow", "Filter", None, -1))
        self.dockLibrary.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Library of components", None, -1))
        self.dockLibrary.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Library design components", None,
                                             -1))
        self.dockLibrary.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Library design components", None,
                                             -1))
        self.dockLibrary.setWindowTitle(
            QtWidgets.QApplication.translate("MainWindow", "Library", None, -1))
        self.dockComponent.setWindowTitle(
            QtWidgets.QApplication.translate("MainWindow", "Edit component",
                                             None, -1))
        self.dockLog.setToolTip(
            QtWidgets.QApplication.translate("MainWindow", "Log window", None,
                                             -1))
        self.dockLog.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow", "Log window", None,
                                             -1))
        self.dockLog.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow", "Log window", None,
                                             -1))
        self.dockLog.setWindowTitle(
            QtWidgets.QApplication.translate("MainWindow", "Log", None, -1))
        self.log_text.setDocumentTitle(
            QtWidgets.QApplication.translate("MainWindow", "Log", None, -1))
        self.dockConnectors.setWindowTitle(
            QtWidgets.QApplication.translate("MainWindow", "Pins", None, -1))
        self.text_filter_connectors.setPlaceholderText(
            QtWidgets.QApplication.translate("MainWindow", "Filter", None, -1))
        self.dockVariables.setWindowTitle(
            QtWidgets.QApplication.translate("MainWindow", "Variables", None,
                                             -1))
        self.toolbar_renderers.setWindowTitle(
            QtWidgets.QApplication.translate("MainWindow", "toolBar", None, -1))
        self.actionSave.setText(
            QtWidgets.QApplication.translate("MainWindow", "Save", None, -1))
        self.actionSave.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Save design to file", None, -1))
        self.actionSave.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Save design to file", None, -1))
        self.actionSave.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Save design to file", None, -1))
        self.actionSave.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Ctrl+S", None, -1))
        self.actionLoad.setText(
            QtWidgets.QApplication.translate("MainWindow", "Load", None, -1))
        self.actionLoad.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Load design from file", None, -1))
        self.actionLoad.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Load design from file", None, -1))
        self.actionLoad.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Load design from file", None, -1))
        self.actionLoad.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Ctrl+O", None, -1))
        self.actionDesign.setText(
            QtWidgets.QApplication.translate("MainWindow", "Select component",
                                             None, -1))
        self.actionDesign.setToolTip(
            QtWidgets.QApplication.translate(
                "MainWindow",
                "Show design window -- select components (show/hide)", None,
                -1))
        self.actionDesign.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Ctrl+D", None, -1))
        self.actionElements.setText(
            QtWidgets.QApplication.translate("MainWindow", "Elements", None,
                                             -1))
        self.actionElements.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Show/hide elements table", None,
                                             -1))
        self.actionLog.setText(
            QtWidgets.QApplication.translate("MainWindow", "Log", None, -1))
        self.actionLog.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Show/hide log window", None, -1))
        self.actionLog.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Ctrl+L", None, -1))
        self.actionNewComponent.setText(
            QtWidgets.QApplication.translate("MainWindow", "Create", None, -1))
        self.actionNewComponent.setToolTip(
            QtWidgets.QApplication.translate(
                "MainWindow",
                "Create new component: Library of new design components you can create",
                None, -1))
        self.actionNewComponent.setStatusTip(
            QtWidgets.QApplication.translate(
                "MainWindow",
                "Create new component: Library of new design components you can create",
                None, -1))
        self.actionNewComponent.setWhatsThis(
            QtWidgets.QApplication.translate(
                "MainWindow",
                "Create new component: Library of new design components you can create",
                None, -1))
        self.actionNewComponent.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Ctrl+N", None, -1))
        self.actionDelete_All.setText(
            QtWidgets.QApplication.translate("MainWindow", "Delete all", None,
                                             -1))
        self.actionDelete_All.setToolTip(
            QtWidgets.QApplication.translate(
                "MainWindow", "Delete All Component Objects and Elements", None,
                -1))
        self.actionDelete_All.setStatusTip(
            QtWidgets.QApplication.translate(
                "MainWindow", "Delete All Component Objects and Elements", None,
                -1))
        self.actionDelete_All.setWhatsThis(
            QtWidgets.QApplication.translate(
                "MainWindow", "Delete All Component Objects and Elements", None,
                -1))
        self.actionZoom.setText(
            QtWidgets.QApplication.translate("MainWindow", "Zoom", None, -1))
        self.actionZoom.setToolTip(
            QtWidgets.QApplication.translate("MainWindow", "Zoom control", None,
                                             -1))
        self.actionZoom.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Z", None, -1))
        self.actionPan.setText(
            QtWidgets.QApplication.translate("MainWindow", "Pan", None, -1))
        self.actionPan.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "P", None, -1))
        self.actionConnectors.setText(
            QtWidgets.QApplication.translate("MainWindow", "Pins", None, -1))
        self.actionConnectors.setToolTip(
            QtWidgets.QApplication.translate(
                "MainWindow", "Show connector pins of qcomponents", None, -1))
        self.actionConnectors.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "C", None, -1))
        self.actionStyleOpen.setText(
            QtWidgets.QApplication.translate("MainWindow", "Open File", None,
                                             -1))
        self.actionStyleDefault.setText(
            QtWidgets.QApplication.translate("MainWindow", "System default",
                                             None, -1))
        self.actionStyleDark.setText(
            QtWidgets.QApplication.translate("MainWindow", "Q Dark", None, -1))
        self.actionScreenshot.setText(
            QtWidgets.QApplication.translate("MainWindow", "Screenshot", None,
                                             -1))
        self.actionScreenshot.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Take a screenshot of the window",
                                             None, -1))
        self.action_full_refresh.setText(
            QtWidgets.QApplication.translate("MainWindow", "Replot", None, -1))
        self.action_full_refresh.setToolTip(
            QtWidgets.QApplication.translate(
                "MainWindow",
                "Replot all components (and refresh all gui widgets). Does NOT REBUILD",
                None, -1))
        self.action_full_refresh.setStatusTip(
            QtWidgets.QApplication.translate(
                "MainWindow", "Force a full refresh of all plots and widgets",
                None, -1))
        self.action_full_refresh.setWhatsThis(
            QtWidgets.QApplication.translate(
                "MainWindow", "Force a full refresh of all plots and widgets",
                None, -1))
        self.action_full_refresh.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Ctrl+R", None, -1))
        self.actionRebuild.setText(
            QtWidgets.QApplication.translate("MainWindow", "Rebuild", None, -1))
        self.actionRebuild.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Force rebuild all components",
                                             None, -1))
        self.actionRebuild.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Force rebuild all components",
                                             None, -1))
        self.actionRebuild.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Force rebuild all components",
                                             None, -1))
        self.actionRebuild.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Ctrl+D", None, -1))
        self.actionFull_Screen.setText(
            QtWidgets.QApplication.translate("MainWindow", "Full Screen", None,
                                             -1))
        self.actionFull_Screen.setToolTip(
            QtWidgets.QApplication.translate("MainWindow", "Go to full screen",
                                             None, -1))
        self.actionMetal_Dark.setText(
            QtWidgets.QApplication.translate("MainWindow", "Metal Dark", None,
                                             -1))
        self.actionSaveAs.setText(
            QtWidgets.QApplication.translate("MainWindow", "Save as", None, -1))
        self.actionSaveAs.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Save design to new file", None,
                                             -1))
        self.actionSaveAs.setStatusTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Save design to file", None, -1))
        self.actionSaveAs.setWhatsThis(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Save design to file", None, -1))
        self.actionSaveAs.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Ctrl+Shift+S", None,
                                             -1))
        self.actionSave_window_state.setText(
            QtWidgets.QApplication.translate("MainWindow", "Save window state",
                                             None, -1))
        self.actionLabelDesign.setText(
            QtWidgets.QApplication.translate("MainWindow", "Design", None, -1))
        self.actionClose_window.setText(
            QtWidgets.QApplication.translate("MainWindow", "Close window", None,
                                             -1))
        self.actionMetal_Window.setText(
            QtWidgets.QApplication.translate("MainWindow", "Metal Window ",
                                             None, -1))
        self.actionViewDummyLabel.setText(
            QtWidgets.QApplication.translate("MainWindow", "View", None, -1))
        self.actionVariables.setText(
            QtWidgets.QApplication.translate("MainWindow", "Design variables",
                                             None, -1))
        self.actionVariables.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Edit variables of the QDesign",
                                             None, -1))
        self.actionVariables.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Meta+V", None, -1))
        self.actionToggleDocks.setText(
            QtWidgets.QApplication.translate("MainWindow", "Toggle view", None,
                                             -1))
        self.actionToggleDocks.setToolTip(
            QtWidgets.QApplication.translate("MainWindow", "ToggleDocks", None,
                                             -1))
        self.actionToggleDocks.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Meta+F, Ctrl+F",
                                             None, -1))
        self.actionGDS.setText(
            QtWidgets.QApplication.translate("MainWindow", "GDS", None, -1))
        self.actionGDS.setToolTip(
            QtWidgets.QApplication.translate("MainWindow",
                                             "Open the GDS renderer.", None,
                                             -1))
        self.actionHFSS.setText(
            QtWidgets.QApplication.translate("MainWindow", "HFSS", None, -1))
        self.actionQ3D.setText(
            QtWidgets.QApplication.translate("MainWindow", "Q3D", None, -1))
        self.actionBuildHistory.setText(
            QtWidgets.QApplication.translate("MainWindow", "Build History",
                                             None, -1))
        self.actionBuildHistory.setToolTip(
            QtWidgets.QApplication.translate(
                "MainWindow", "See the build history of your design", None, -1))
        self.actionBuildHistory.setShortcut(
            QtWidgets.QApplication.translate("MainWindow", "Ctrl+B", None, -1))
        self.actionDeveloperMode.setText(
            QtWidgets.QApplication.translate("MainWindow", "DeveloperMode",
                                             None, -1))
        self.actionDeveloperMode.setToolTip(
            QtWidgets.QApplication.translate("MainWindow", "clickme", None, -1))
        self.actionWebHelp.setText(
            QtWidgets.QApplication.translate("MainWindow", "Web Help", None,
                                             -1))
        self.actionWebHelp.setToolTip(
            QtWidgets.QApplication.translate("MainWindow", "Open Web Help",
                                             None, -1))


from .widgets.all_components.table_view_all_components import QTableView_AllComponents
from .widgets.log_widget.log_metal import QTextEditLogger
from .widgets.qlibrary_display.tree_view_qlibrary import TreeViewQLibrary
from . import main_window_rc_rc



# -*- coding: utf-8 -*-

# This code is part of Qiskit.
#
# (C) Copyright IBM 2017, 2021.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.

# pylint: disable=wrong-import-order
# pylint: disable=wrong-import-position

# Get Qiskit Metal version from metadata without importing the package
from importlib.metadata import version

"""Qiskit Metal"""
__version__ = version('quantum-metal')
__license__ = "Apache 2.0"
__copyright__ = 'Copyright IBM 2019-2020'
__author__ = 'Zlatko Minev, Thomas McConkey, and them IBM Quantum Team'
__status__ = "Development"

###########################################################################
### Windows OS catch for library geopandas not installed with setup.py
### TODO: Check if this is still needed with uv_build.

import os
if os.name == 'nt':
    try:
        import geopandas
    except ImportError:
        print(" \
            QISKIT METAL INFORMATION: >>>>>>>>>> One last installation step. <<<<<<<<<<<\n \
            >>>>>> Packages fiona and gdal have a known install issue on Windows. <<<<<<\n \
            >>>>>>>>>> geopandas depends on fiona, and fiona depends on gdal. <<<<<<<<<<\n \
            >> To prevent the unavoidable pip installation fail, we excluded geopandas <\n \
            >>>>>> from the list of qiskit-metal package dependencies in Windows. <<<<<<\n \
            >>>>>>>> Before you can use Qiskit Metal, please install geopandas. <<<<<<<<\n \
            >>>> For more information, you can follow the instructions on this FAQ <<<<<\n \
            >>>>>>>>>>>>> https://qiskit-community.github.io/qiskit-metal/faq.html <<<<<<<<<<<<<<\n"
              )
        raise


###########################################################################
### Basic Setups
## Setup Qt
def __setup_Qt_backend():  # pylint: disable=invalid-name
    """Setup matplotlib to use Qt5's visualization.

    This function needs to remain in the __init__ of the library's root
    to prevent Qt windows from hanging.
    """
    # pylint: disable=import-outside-toplevel

    # When in vscode and in debug-mode, may want to comment
    # next line out, "os.environ["QT_API"] = "pyside2""
    os.environ["QT_API"] = "pyside6"

    from PySide6 import QtCore  #, QtWidgets
    from PySide6.QtCore import Qt

    def set_attribute(name: str, value=True):
        """Describes attributes that change the behavior of application-wide
        features."""
        if hasattr(Qt, name):
            # Does Qt have this attribute
            attr = getattr(Qt, name)
            if not QtCore.QCoreApplication.testAttribute(attr) == value:
                # Only set if not already set
                QtCore.QCoreApplication.setAttribute(attr, value)

    if 1:

        if QtCore.QCoreApplication.instance() is None:
            # No application launched yet

            # zkm: The following seems to fix warning.
            # For example if user ran %gui qt already.
            #  Qt WebEngine seems to be initialized from a plugin.
            # Please set Qt::AA_ShareOpenGLContexts using QCoreApplication::setAttribute
            #  before constructing QGuiApplication.
            # https://stackoverflow.com/questions/56159475/qt-webengine-seems-to-be-initialized
            # Enables resource sharing between the OpenGL contexts used by classes
            #  like QOpenGLWidget and QQuickWidget.
            # Has to do with render mode 'gles'. There is also desktop and software.
            # QCoreApplication.setAttribute(QtCore.Qt.AA_UseOpenGLES)
            # QCoreApplication.setAttribute(QtCore.Qt.AA_ShareOpenGLContexts)
            # QCoreApplication.setAttribute(QtCore.Qt.AA_DisableShaderDiskCache)
            set_attribute('AA_ShareOpenGLContexts')

            # Enables high-DPI scaling in Qt on supported platforms (see also High DPI Displays).
            # Supported platforms are X11, Windows and Android.
            # Enabling makes Qt scale the main (device independent) coordinate
            # system according to display scale factors provided by the
            # operating system.
            set_attribute('AA_EnableHighDpiScaling')

            # Make QIcon::pixmap() generate high-dpi pixmaps that can be larger than
            #  the requested size.
            set_attribute('AA_UseHighDpiPixmaps')

            # Other options of interest:
            # AA_DontUseNativeMenuBar
            # AA_MacDontSwapCtrlAndMeta

    if not os.getenv('QISKIT_METAL_HEADLESS', None):
        # pylint: disable=import-outside-toplevel
        import matplotlib as mpl
        mpl.use("QtAgg")
        # pylint: disable=redefined-outer-name
        import matplotlib.pyplot as plt
        plt.ion()  # interactive


__setup_Qt_backend()
del __setup_Qt_backend

## Setup logging
from . import config
from .toolbox_python._logging import setup_logger

logger = setup_logger('metal',
                      config.log.format,
                      config.log.datefmt,
                      capture_warnings=True)  # type: logging.Logger
del setup_logger

###########################################################################
### User-accessible scope

# Metal Dict
from .toolbox_python.attr_dict import Dict

# Due to order of imports
from ._is_design import is_design, is_component

# Core modules for user to use
from .toolbox_metal.parsing import is_true
from . import qlibrary
from . import designs
from . import draw
from . import renderers
from . import qgeometries
from . import analyses
from . import toolbox_python
from . import toolbox_metal

# Metal GUI
from ._gui.main_window import MetalGUI

# Utility modules
# For plotting in matplotlib;  May be superseded by a renderer?
from .renderers.renderer_mpl import mpl_toolbox as plt

# Utility functions
from .toolbox_python.display import Headings

# Import default renderers
from .renderers import setup_renderers

# Common-use
from .qlibrary import QComponent
from .toolbox_metal.about import about, open_docs



# -*- coding: utf-8 -*-

# This code is part of Qiskit.
#
# (C) Copyright IBM 2017, 2021.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.

#pylint: disable-msg=unnecessary-pass
#pylint: disable-msg=broad-except
"""Qiskit Metal unit tests analyses functionality.

Test a planar design and launching the GUI.
"""

import unittest
from qiskit_metal._gui.widgets.bases.dict_tree_base import BranchNode
from qiskit_metal._gui.widgets.bases.dict_tree_base import LeafNode


class TestGUIBasic(unittest.TestCase):
    """Unit test class."""

    def setUp(self):
        """Setup unit test."""
        pass

    def tearDown(self):
        """Tie any loose ends."""
        pass

    def test_instantiate_branch_node(self):
        """Test instantiation of BranchNode in dict_tree_base.py."""
        try:
            BranchNode('my_name')
        except Exception:
            message = "BranchNode instantiation failed"
            self.fail(message)

    def test_instantiate_leaf_node(self):
        """Test instantiation of LeafNode in dict_tree_base.py."""
        try:
            LeafNode('my_label')
        except Exception:
            message = "LeafNode instantiation failed"
            self.fail(message)


if __name__ == '__main__':
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-

# This code is part of Qiskit.
#
# (C) Copyright IBM 2017, 2021.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.
"""
=================================================
Renderers (:mod:`qiskit_metal.renderers`)
=================================================

.. currentmodule:: qiskit_metal.renderers

.. _qrenderer:

Renderer Base
---------------

.. autosummary::
    :toctree: .

    QRenderer
    QRendererGui
    QRendererAnalysis


MPL Renderer
---------------

.. autosummary::
    :toctree: .

    PlotCanvas
    MplInteraction
    ZoomOnWheel
    PanAndZoom
    QMplRenderer
    AnimatedText


GDS Renderer
---------------

.. autosummary::
    :toctree: .

    QGDSRenderer
    Cheesing


Ansys Renderer
---------------

.. autosummary::
    :toctree: .

    QAnsysRenderer
    QHFSSRenderer
    QQ3DRenderer
    QPyaedt
    QQ3DPyaedt
    QHFSSPyaedt
    QHFSSDrivenmodalPyaedt
    QHFSSEigenmodePyaedt



GMSH Renderer
---------------

.. autosummary::
    :toctree: .

    QGmshRenderer
    Vec3DArray



"""

from .setup_default import setup_renderers

from .. import config
if config.is_building_docs():
    from .renderer_base.renderer_base import QRenderer
    from .renderer_base.renderer_gui_base import QRendererGui
    from .renderer_base.rndr_analysis import QRendererAnalysis

    from .renderer_gds.gds_renderer import QGDSRenderer
    from .renderer_gds.make_cheese import Cheesing

    from .renderer_mpl.mpl_canvas import PlotCanvas
    from .renderer_mpl.mpl_interaction import MplInteraction
    from .renderer_mpl.mpl_interaction import ZoomOnWheel
    from .renderer_mpl.mpl_interaction import PanAndZoom
    from .renderer_mpl.mpl_renderer import QMplRenderer
    from .renderer_mpl.extensions.animated_text import AnimatedText

    from .renderer_mpl import mpl_interaction
    from .renderer_mpl import mpl_toolbox

    from .renderer_ansys.ansys_renderer import QAnsysRenderer
    from .renderer_ansys.hfss_renderer import QHFSSRenderer
    from .renderer_ansys.q3d_renderer import QQ3DRenderer

    from .renderer_gmsh.gmsh_utils import Vec3DArray
    from .renderer_gmsh.gmsh_renderer import QGmshRenderer

    from .renderer_ansys_pyaedt.pyaedt_base import QPyaedt
    from .renderer_ansys_pyaedt.q3d_renderer_aedt import QQ3DPyaedt
    from .renderer_ansys_pyaedt.hfss_renderer_aedt import QHFSSPyaedt
    from .renderer_ansys_pyaedt.hfss_renderer_drivenmodal_aedt import QHFSSDrivenmodalPyaedt
    from .renderer_ansys_pyaedt.hfss_renderer_eigenmode_aedt import QHFSSEigenmodePyaedt



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'c:\Temp\GitHub\qiskit-metal\qiskit_metal\_gui\renderer_gds_ui.ui'
#
# Created: Sat Jun 19 22:02:30 2021
#      by: pyside6-uic  running on PySide2 5.13.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.

from PySide6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):

    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(651, 581)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.instructionsLabel = QtWidgets.QLabel(self.centralwidget)
        self.instructionsLabel.setObjectName("instructionsLabel")
        self.gridLayout.addWidget(self.instructionsLabel, 0, 0, 1, 1)
        self.instructionsLabel_2 = QtWidgets.QLabel(self.centralwidget)
        self.instructionsLabel_2.setAlignment(QtCore.Qt.AlignCenter)
        self.instructionsLabel_2.setObjectName("instructionsLabel_2")
        self.gridLayout.addWidget(self.instructionsLabel_2, 0, 1, 1, 1)
        self.listView = QtWidgets.QListView(self.centralwidget)
        self.listView.setObjectName("listView")
        self.gridLayout.addWidget(self.listView, 1, 0, 2, 1)
        self.treeView = QTreeView_Base(self.centralwidget)
        self.treeView.setRootIsDecorated(False)
        self.treeView.setObjectName("treeView")
        self.gridLayout.addWidget(self.treeView, 1, 1, 1, 1)
        self.label = QtWidgets.QLabel(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred,
                                           QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 2, 1, 2, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.refreshButton = QtWidgets.QPushButton(self.centralwidget)
        self.refreshButton.setObjectName("refreshButton")
        self.horizontalLayout_4.addWidget(self.refreshButton)
        self.selectAllButton = QtWidgets.QPushButton(self.centralwidget)
        self.selectAllButton.setObjectName("selectAllButton")
        self.horizontalLayout_4.addWidget(self.selectAllButton)
        self.deselectAllButton = QtWidgets.QPushButton(self.centralwidget)
        self.deselectAllButton.setObjectName("deselectAllButton")
        self.horizontalLayout_4.addWidget(self.deselectAllButton)
        self.gridLayout.addLayout(self.horizontalLayout_4, 3, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lineEdit = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding,
                                           QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.lineEdit.sizePolicy().hasHeightForWidth())
        self.lineEdit.setSizePolicy(sizePolicy)
        self.lineEdit.setObjectName("lineEdit")
        self.horizontalLayout.addWidget(self.lineEdit)
        self.browseButton = QtWidgets.QToolButton(self.centralwidget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/_imgs/search.png"), QtGui.QIcon.Normal,
                       QtGui.QIcon.Off)
        self.browseButton.setIcon(icon)
        self.browseButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.browseButton.setAutoRaise(False)
        self.browseButton.setObjectName("browseButton")
        self.horizontalLayout.addWidget(self.browseButton)
        self.exportButton = QtWidgets.QPushButton(self.centralwidget)
        self.exportButton.setObjectName("exportButton")
        self.horizontalLayout.addWidget(self.exportButton)
        self.gridLayout.addLayout(self.horizontalLayout, 4, 0, 1, 2)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 651, 18))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        self.browseButton.clicked.connect(
            MainWindow.browse_folders)  # type: ignore
        self.exportButton.clicked.connect(
            MainWindow.export_file)  # type: ignore
        self.selectAllButton.clicked.connect(
            MainWindow.select_all)  # type: ignore
        self.deselectAllButton.clicked.connect(
            MainWindow.deselect_all)  # type: ignore
        self.refreshButton.clicked.connect(MainWindow.refresh)  # type: ignore
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "GDS Renderer"))
        self.instructionsLabel.setText(
            _translate("MainWindow", "Check off components to export:"))
        self.instructionsLabel_2.setText(
            _translate("MainWindow", "Renderer options"))
        self.label.setText(
            _translate(
                "MainWindow",
                "Note: The user-defined bounding box scales above are only relevant when not all components are being exported. Otherwise, the bounding box for the subtraction layer is obtained from the DesignPlanar class."
            ))
        self.refreshButton.setText(_translate("MainWindow", "Refresh List"))
        self.selectAllButton.setText(_translate("MainWindow", "Select All"))
        self.deselectAllButton.setText(_translate("MainWindow", "Deselect All"))
        self.lineEdit.setPlaceholderText(
            _translate("MainWindow",
                       "Export GDS to the following location... "))
        self.browseButton.setText(_translate("MainWindow", "Browse"))
        self.exportButton.setText(_translate("MainWindow", "Export"))


from . import main_window_rc_rc
from .tree_view_base import QTreeView_Base



# -*- coding: utf-8 -*-

# This code is part of Qiskit.
#
# (C) Copyright IBM 2017, 2021.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.

from qiskit_metal import Dict
from qiskit_metal.qlibrary import QRoute


class RouteStraight(QRoute):
    """
    Draw a straight Route connecting two pins.
    
    .. meta::
        :description: Route Straight
    """

    component_metadata = Dict(short_name='cpw')
    """Component metadata"""

    TOOLTIP = """Draw a straight Route connecting two pins."""

    def make(self):
        """The make function implements the logic that creates the geometry
        (poly, path, etc.) from the qcomponent.options dictionary of
        parameters, and the adds them to the design, using
        qcomponent.add_qgeometry(...), adding in extra needed information, such
        as layer, subtract, etc."""
        # Set the CPW pins and add the points/directions to the lead-in/out arrays
        self.set_pin("start")
        self.set_pin("end")

        # Align the lead-in/out to the input options set from the user
        self.set_lead("start")
        self.set_lead("end")

        # Make points into elements
        self.make_elements(self.get_points())



from dataclasses import dataclass, field
from typing import Union, List, Tuple
import numpy as np
import gmsh
from qiskit_metal.draw.utility import Vec3D


@dataclass
class Vec3DArray:
    """Class to define an array of np.ndarray objects

    Raises:
        TypeError: if layer_z value is not provided for 2D design
        ValueError: if dimensionality of coordinate > 3

    Returns:
        Vec3DArray: Array of np.ndarray objects
    """
    points: List[np.ndarray]
    path_vecs: List[np.ndarray] = field(init=False)

    def __post_init__(self):
        """This is to initialize the `path_vecs` field separately
        as it depends on the `points` field, and needs to be
        calculated after the __init__ for the dataclass is called.

        CAUTION: Please don't change this unless you really need to!
        """
        self.path_vecs = []
        for i in range(0, len(self.points) - 1):
            v1 = self.points[i]
            v2 = self.points[i + 1]
            v12 = Vec3D.normed(Vec3D.sub(v2, v1))
            self.path_vecs.append(v12)

    def append(self, vecs: List[np.ndarray]):
        """Append vector to the array

        Args:
            vecs (List[np.ndarray]): list of vectors to append
        """
        self.points += vecs

    def get_angle_between(self,
                          i: int,
                          j: int,
                          ret_path_angle: bool = True) -> float:
        """Return acute angle between two vectors depending on "ret_path_angle"

        Args:
            i (int): i^th index from path_vecs or points list
            j (int): j^th index from path_vecs or points list
            ret_path_angle (bool, optional): Return angl between path_vecs
                    if True and between points if False. Defaults to True.

        Returns:
            float: angle between the corresponding vectors
        """
        if ret_path_angle:
            v1 = Vec3D.normed(self.path_vecs[i])
            v2 = Vec3D.normed(self.path_vecs[j])
            return np.round(np.pi - np.arccos(Vec3D.dot(v1, v2)), decimals=9)

        v1 = Vec3D.normed(self.points[i])
        v2 = Vec3D.normed(self.points[j])
        return np.round(np.pi - np.arccos(Vec3D.dot(v1, v2)), decimals=9)

    @staticmethod
    def make_vec3DArray(points: List[List[Union[int, float]]],
                        layer_z: float = None):
        """Create a Vec3DArray object from list of points

        Args:
            points (List[List[Union[int, float]]]): list of points
            layer_z (float, optional): z-coordinate of layer in case
                                    of 2D design. Defaults to None.

        Raises:
            TypeError: if layer_z value is not provided for 2D design
            ValueError: if dimensionality of coordinate > 3

        Returns:
            Vec3DArray: return a Vec3DArray object
        """
        vecs = []
        for xyz in points:
            if len(xyz) > 2:
                vecs.append(np.array(xyz))
            elif len(xyz) == 2:
                if layer_z is not None:
                    vecs.append(np.array([xyz[0], xyz[1], layer_z]))
                else:
                    raise TypeError(
                        f"Expected a layer_z value for 2D geometry, found NoneType."
                    )
            else:
                raise ValueError(
                    f"Expected a 2-D or 3-D coordinate, got {len(xyz)}-D instead."
                )

        return Vec3DArray(vecs)


def vecs_to_gmsh_points(vecs: List[np.ndarray], layer_z: float) -> list:
    """Create Gmsh points from np.ndarray objs

    Args:
        vecs (List[np.ndarray]): list of np.ndarray objs
        layer_z (float): z-coordinate of layer

    Returns:
        list: list of Gmsh points
    """
    points = []
    for v in vecs:
        p = gmsh.model.occ.addPoint(v[0], v[1], layer_z)
        points.append(p)

    return points


def line_width_offset_pts(pt_vec: np.ndarray,
                          path_vec: np.ndarray,
                          width: float,
                          layer_z: float,
                          ret_pts: bool = True) -> list:
    """Create offset points for straight line

    Args:
        pt_vec (np.ndarray): vectors of points in line
        path_vec (np.ndarray): vectors along lines
        width (float): width for offset
        layer_z (float): z-coordinate of layer
        ret_pts (bool, optional): Return Gmsh points if True
                        else return np.ndarray. Defaults to True.

    Returns:
        list: list of Gmsh points or vec3D objs
    """
    path_angle = Vec3D.angle_azimuth(path_vec)
    perp_angle = path_angle + np.pi / 2
    cos_t = np.round(np.cos(perp_angle), decimals=9)
    sin_t = np.round(np.sin(perp_angle), decimals=9)
    r = width / 2

    offset_vec = np.array([r * cos_t, r * sin_t, 0])
    v1 = Vec3D.add(pt_vec, offset_vec)
    v2 = Vec3D.sub(pt_vec, offset_vec)

    if ret_pts:
        pts = vecs_to_gmsh_points([v1, v2], layer_z)
        return pts
    else:
        return [v1, v2]


def arc_width_offset_pts(vec1: np.ndarray, vec3: np.ndarray, angle: float,
                         width: float, layer_z: float) -> list:
    """Create offset points for Circle Arcs

    Args:
        vec1 (np.ndarray): incoming vector to arc
        vec3 (np.ndarray): outgoing vector from arc
        angle (float): angle of arc
        width (float): width for offset
        layer_z (float): z-coordinate of layer

    Returns:
        list: list of Gmsh points
    """
    perp_angle = np.pi / 2 - angle  # π - (angle + π/2)
    cos_t = np.round(np.cos(perp_angle), decimals=9)
    sin_t = np.round(np.sin(perp_angle), decimals=9)
    r = width / 2

    offset1 = np.array([0, r, 0])
    v1 = Vec3D.add(vec1, offset1)
    v2 = Vec3D.sub(vec1, offset1)

    sign = 1 if angle == np.pi / 2 else -1
    offset2 = np.array([r * cos_t * sign, r * sin_t * sign, 0])

    v3 = Vec3D.add(vec3, offset2)
    v4 = Vec3D.sub(vec3, offset2)

    pts = vecs_to_gmsh_points([v1, v2, v3, v4], layer_z)
    return pts


def make_arc_vecs(angle: float,
                  fillet: float) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
    """Create vectors for the circle arc

    Args:
        angle (float): angle of arc
        fillet (float): fillet radius

    Returns:
        Tuple[np.ndarray, np.ndarray, np.ndarray]: vectors defining the arc control points
    """
    sector_angle = np.pi - angle
    intercept = np.abs(fillet * np.tan(sector_angle / 2))
    cos_t = np.round(np.cos(sector_angle), decimals=9)
    sin_t = np.round(np.sin(sector_angle), decimals=9)

    p1x = np.round(-intercept, decimals=9)
    p1y = 0
    p2x = np.round(p1x, decimals=9)
    p2y = np.round(p1y + fillet, decimals=9)
    p3x = np.round(intercept * cos_t, decimals=9)
    p3y = np.round(intercept * sin_t, decimals=9)

    v1 = np.array([p1x, p1y, 0])
    v2 = np.array([p2x, p2y, 0])
    v3 = np.array([p3x, p3y, 0])

    return v1, v2, v3


def transform_arc_points(pts: list, translate: np.ndarray,
                         path_vecs: list) -> list:
    """Apply transformation to arc points

    Args:
        pts (list): list of Gmsh points
        translate (np.ndarray): translation vector
        path_vecs (list): vectors of actual position

    Returns:
        list: list of Gmsh points
    """
    dim_tags = [(0, p) for p in pts]
    new_pts = [p for p in pts]

    angle1 = Vec3D.angle_azimuth(path_vecs[0])
    cross_vec = Vec3D.cross(path_vecs[0], path_vecs[1])
    mirror = True if np.sign(Vec3D.normed(cross_vec)[2]) < 0 else False

    if mirror:
        # Flip right turn start/end points
        p1 = new_pts.pop(0)
        p4 = new_pts.pop(2)
        new_pts.insert(1, p1)
        new_pts.append(p4)
        gmsh.model.occ.mirror(dim_tags, a=0, b=1, c=0, d=0)

    gmsh.model.occ.rotate(dim_tags,
                          x=0,
                          y=0,
                          z=0,
                          ax=0,
                          ay=0,
                          az=1,
                          angle=angle1)
    gmsh.model.occ.translate(dim_tags, translate[0], translate[1], 0)
    return new_pts


def draw_curves(recent_pts: list, curves1: list,
                curves2: list) -> Tuple[list, list, list]:
    """Draw the curves using control points

    Args:
        recent_pts (list): list to keep track of recent points
        curves1 (list): curves on one side of offset
        curves2 (list): curves on other side of offset

    Raises:
        RuntimeError: raised when the number of points in recent_pts
                            is not what is expected by the program.

    Returns:
        Tuple[list, list, list]: returns a tuple of recent_pts, curves1, curves2
    """
    rec_pts = []

    if len(recent_pts) not in [4, 7]:
        raise RuntimeError(
            "Unexpected geometry {len(recent_pts) not in [4,7]}. Check your geometry and retry."
        )

    l1 = gmsh.model.occ.addLine(recent_pts[0], recent_pts[2])
    l2 = gmsh.model.occ.addLine(recent_pts[1], recent_pts[3])
    curves1 += [l1]
    curves2 += [l2]

    if len(recent_pts) == 7:
        a1 = gmsh.model.occ.addCircleArc(recent_pts[2], recent_pts[4],
                                         recent_pts[5])
        a2 = gmsh.model.occ.addCircleArc(recent_pts[3], recent_pts[4],
                                         recent_pts[6])
        curves1 += [a1]
        curves2 += [a2]

    rec_pts = recent_pts[-2:]

    return rec_pts, curves1, curves2


def render_path_curves(vecs: Vec3DArray,
                       layer_z: float,
                       fillet: float,
                       width: float,
                       bad_fillet_idxs: List[int],
                       straight_line_tol: float = 1e-9) -> list:
    """Helper function for rendering path QGeometry curves

    Args:
        vecs (Vec3DArray): vector array
        layer_z (float): z-coordinate of layer
        fillet (float): fillet radius
        width (float): width for offset
        straight_line_tol (float, optional): Tolerance for straight line
                                        through a point. Defaults to 1e-9.

    Raises:
        ValueError: raises when fillet radius < 0.0

    Returns:
        list: list of Gmsh curves
    """
    recent_pts = []  # To store recent Gmsh points
    curves1, curves2 = [], []  # To store lines and arcs for each visual side

    for i, v in enumerate(vecs.points):
        if i == 0:
            p1, p2 = line_width_offset_pts(v, vecs.path_vecs[i], width, layer_z)
            recent_pts += [p1, p2]
            curves1 += [gmsh.model.occ.addLine(p1, p2)]
            continue

        if i == len(vecs.points) - 1:
            p1, p2 = line_width_offset_pts(v, vecs.path_vecs[i - 1], width,
                                           layer_z)
            recent_pts += [p1, p2]
            curves1 += [gmsh.model.occ.addLine(p1, p2)]
            continue

        pv1 = vecs.path_vecs[i - 1]
        pv2 = vecs.path_vecs[i]
        angle12 = vecs.get_angle_between(i - 1, i)
        is_filleted = False if i in bad_fillet_idxs else True

        if fillet < 0.0:
            raise ValueError(f"Expected positive fillet radius, got {fillet}.")

        elif fillet > 0.0 and is_filleted:
            if np.allclose(angle12, np.pi, rtol=straight_line_tol):
                p1, p2 = line_width_offset_pts(v, pv2, width, layer_z)
                recent_pts += [p1, p2]

            else:
                v1, v2, v3 = make_arc_vecs(angle12, fillet)
                p1, p2, p4, p5 = arc_width_offset_pts(v1, v3, angle12, width,
                                                      layer_z)
                p3 = vecs_to_gmsh_points([v2], layer_z)[0]

                new_pts = transform_arc_points([p1, p2, p3, p4, p5], v,
                                               [pv1, pv2])
                recent_pts += new_pts

        else:
            half_angle = angle12 / 2
            if np.allclose(angle12, np.pi, rtol=straight_line_tol):
                right_turn = 1
            else:
                right_turn = -np.sign(Vec3D.normed(Vec3D.cross(pv1, pv2))[2])
            turn_pt_vec = vecs.points[i]
            pv1_normed = Vec3D.normed(pv1)
            pv1_scaled = Vec3D.scale(
                pv1_normed, width / (2 * np.cos(np.pi / 2 - half_angle)))
            pv1_trans = Vec3D.translate(pv1_scaled, turn_pt_vec)
            offset_vec1 = Vec3D.rotate(pv1_trans,
                                       turn_pt_vec,
                                       az=True,
                                       radians=right_turn * half_angle)
            offset_vec2 = Vec3D.rotate(pv1_trans,
                                       turn_pt_vec,
                                       az=True,
                                       radians=-right_turn *
                                       (np.pi - half_angle))
            p1 = gmsh.model.occ.addPoint(offset_vec1[0], offset_vec1[1],
                                         layer_z)
            p2 = gmsh.model.occ.addPoint(offset_vec2[0], offset_vec2[1],
                                         layer_z)
            recent_pts += [p1, p2] if right_turn > 0 else [p2, p1]

        recent_pts, curves1, curves2 = draw_curves(recent_pts, curves1, curves2)

    _, curves1, curves2 = draw_curves(recent_pts, curves1, curves2)

    last_curve = curves1.pop(-2)
    curves1.append(last_curve)
    curves = curves1 + curves2[::-1]

    return curves



from qiskit_metal import draw, Dict
from qiskit_metal.qlibrary.core import QComponent
import numpy as np


class QuadCoupler(QComponent):
    """Generates a one pin (+) structure comprised of a rectangular coupling pad
    capacitively coupled with any other component.

    .. meta::
        General Quadrilateral Coupler
    """

    component_metadata = Dict(short_name='cpw', _qgeometry_table_path='True')
    """Component metadata"""

    default_options = Dict(pad_width='40um',
                           pad_height='60um',
                           pad_gap='6um',
                           cpw_stub_width='10um',
                           cpw_stub_height='30um',
                           fillet='25um')
    """Default connector options"""

    def make(self):
        p = self.p

        coupler_pad = draw.rectangle(p.pad_width, p.pad_height)
        coupler_sub_pad = draw.rectangle(p.pad_width + p.pad_gap * 2,
                                         p.pad_height + p.pad_gap * 2)

        yoff = (p.pad_height + p.cpw_stub_height) / 2
        cpw_conn = draw.LineString([[0, -p.pad_height / 2],
                                    [0, -p.pad_height / 2 - p.cpw_stub_height]])
        cpw_conn_sub = draw.LineString(
            [[0, -p.pad_height / 2], [0,
                                      -p.pad_height / 2 - p.cpw_stub_height]])

        items = [coupler_pad, coupler_sub_pad, cpw_conn, cpw_conn_sub]
        items = draw.rotate(items, p.orientation, origin=(0, 0))
        items = draw.translate(items, p.pos_x, p.pos_y)
        coupler_pad, coupler_sub_pad, cpw_conn, cpw_conn_sub = items

        self.add_qgeometry('poly', {'quad_coupler': coupler_pad}, layer=p.layer)
        self.add_qgeometry('poly', {'quad_coupler_sub': coupler_sub_pad},
                           layer=p.layer,
                           subtract=True)

        self.add_qgeometry('path', {'stub': cpw_conn},
                           layer=p.layer,
                           width=p.cpw_stub_width)
        self.add_qgeometry('path', {'stub_sub': cpw_conn_sub},
                           layer=p.layer,
                           width=p.cpw_stub_width + p.pad_gap * 2,
                           subtract=True)

        pin_list = cpw_conn.coords
        self.add_pin('in',
                     points=np.array(pin_list),
                     width=p.cpw_stub_width,
                     input_as_norm=True)



# -*- coding: utf-8 -*-

# This code is part of Qiskit.
#
# (C) Copyright IBM 2017, 2021.
#
# This code is licensed under the Apache License, Version 2.0. You may
# obtain a copy of this license in the LICENSE.txt file in the root directory
# of this source tree or at http://www.apache.org/licenses/LICENSE-2.0.
#
# Any modifications or derivative works of this code must retain this
# copyright notice, and modified files need to carry a notice indicating
# that they have been altered from the originals.
"""
=================================================
GUI (:mod:`qiskit_metal._gui`)
=================================================

.. currentmodule:: qiskit_metal._gui

GUI module, handles user interface.

The gui module is only loaded if right python qt module
(such as pyside or pyqt) can be found.


Main Window
---------------

.. autosummary::
    :toctree: .

    MetalGUI

"""
# pylint: disable=invalid-name

import logging
from .. import __version__

from .. import config
if config.is_building_docs():
    # imported here for the docstrings
    from qiskit_metal._gui.list_model_base import DynamicList
    from qiskit_metal._gui.main_window_base import QMainWindowExtensionBase
    from qiskit_metal._gui.main_window_base import QMainWindowBaseHandler
    from qiskit_metal._gui.main_window import QMainWindowExtension
    from qiskit_metal._gui.main_window import MetalGUI
    from qiskit_metal._gui.tree_view_base import QTreeView_Base
    from qiskit_metal._gui.renderer_gds_gui import RendererGDSWidget
    from qiskit_metal._gui.renderer_gds_model import RendererGDS_Model
    from qiskit_metal._gui.elements_window import ElementsWindow
    from qiskit_metal._gui.elements_window import ElementTableModel
    from qiskit_metal._gui.widgets.all_components.table_view_all_components import QTableView_AllComponents
    from qiskit_metal._gui.widgets.all_components.table_model_all_components import QTableModel_AllComponents
    from qiskit_metal._gui.widgets.bases.dict_tree_base import BranchNode
    from qiskit_metal._gui.widgets.bases.dict_tree_base import LeafNode
    from qiskit_metal._gui.widgets.bases.dict_tree_base import QTreeModel_Base
    from qiskit_metal._gui.widgets.bases.QWidget_PlaceholderText import QWidget_PlaceholderText
    from qiskit_metal._gui.widgets.edit_component.component_widget import ComponentWidget
    from qiskit_metal._gui.widgets.edit_component.table_model_options import QTableModel_Options
    from qiskit_metal._gui.widgets.edit_component.table_view_options import QTableView_Options
    from qiskit_metal._gui.widgets.edit_component.tree_model_options import QTreeModel_Options
    from qiskit_metal._gui.widgets.edit_component.tree_view_options import QTreeView_Options
    from qiskit_metal._gui.widgets.log_widget.log_metal import QTextEditLogger
    from qiskit_metal._gui.widgets.log_widget.log_metal import LogHandler_for_QTextLog
    from qiskit_metal._gui.widgets.plot_widget.plot_window import QMainWindowPlot
    from qiskit_metal._gui.widgets.variable_table.prop_val_table_gui import PropertyTableWidget
    from qiskit_metal._gui.widgets.variable_table.prop_val_table_model import PropValTable
    from qiskit_metal._gui.widgets.variable_table.right_click_table_view import RightClickView

    from qiskit_metal._gui.utility import _handle_qt_messages
    from qiskit_metal._gui.utility import _toolbox_qt
    from qiskit_metal._gui.widgets.bases import dict_tree_base
    from qiskit_metal._gui.widgets.edit_component import component_widget
    from qiskit_metal._gui.widgets.edit_component import table_model_options
    from qiskit_metal._gui.widgets.edit_component import tree_model_options
